

from .sim_model import SimModel
from .i_sim_model import ISimModel
from .sim_array import SimArray
from .sim_array_view import SimArrayView
from .sim_pstudy import SimPStudy
from .sim_output import SimOut



'''
Created on Aug 8, 2009

@author: alex
'''
import unittest

from traits.api import \
    Instance

import os

from shutil import \
    copy

from .ex_run_view import \
    ExRunView

from matresdev.db.matdb.trc.composite_cross_section \
    import plain_concrete

from matresdev.db.simdb import \
    SimDB

# Access to the toplevel directory of the database
#
simdb = SimDB()


#--------------------------------------------------------------------------------
# run a test cycle of loading - saving - loading
#--------------------------------------------------------------------------------

#----------------------------------------------------------
# define the path to a tensile test:
#----------------------------------------------------------
# - two tests have the same cross section
# - one test has a different cross section and different layup

ex_path_TT_7a_V1 = os.path.join(simdb.exdata_dir, 'tensile_tests',
                                'butstrap_clamping', 'TT-7a',
                                'TT08-7a-V1.DAT')

ex_path_TT_7a_V2 = os.path.join(simdb.exdata_dir, 'tensile_tests', 'TT-7a',
                                'TT08-7a-V2.DAT')


class TestExRunView(unittest.TestCase):
    '''test for a cycle of different show cases.
    '''

    def setUp(self):
        '''open a file for testing and construct
        ExRunView as a class variable.
        '''
        print('\n')
        print('#------------------------------------------')
        print('# SetUp')
        print('#------------------------------------------')

        # define data file (original data file)
        #
        self.data_file_orig_1 = ex_path_TT_7a_V1
        self.data_file_orig_2 = ex_path_TT_7a_V2

        # make a copy of the original data files and get new path of test files
        #
        self.data_file_test_1 = self.copy_data_files(self.data_file_orig_1)
        self.data_file_test_2 = self.copy_data_files(self.data_file_orig_2)

        # reset/setup
        #
        self.exrv = Instance(ExRunView)
        self.delete_pickle_file(self.data_file_test_1)
        self.construct_exrv(self.data_file_test_1)

        # save the default settings
        #
        self.save_run()


    def test_ex_run_view_cycle(self):
        '''test the behavior of ExRunView with respect
        to changes of the input variables for the both cases
        that ExRunView was newly constructed or read in from
        a stored pickle file.
        '''

        # test changes in cm, flu_list and s_tex_z
        # before and after reloading from pickle file:
        #
        for attr in [ 'cm_key', 'age', 's_tex_z', 'flu_list' ]:
            print('\n')
            print('#------------------------------------------')
            print('# ', attr)
            print('#------------------------------------------')
            old_value = self.get_attr(attr)
            print('old_value', old_value)
            new_value = self.change_attr(attr)
            print('new_value', new_value)
#            self.assertNotEqual( old_value, new_value )
            self.assertEqual(self.exrv.unsaved, True)
            self.save_run()

            self.assertEqual(self.exrv.unsaved, False)
            self.reload_data_file(self.data_file_test_1, self.data_file_test_2)
            self.assertEqual(self.exrv.unsaved, False)
            old_value = self.get_attr(attr)
            new_value = self.change_attr(attr)
#            self.assertNotEqual( old_value, new_value )
            self.assertEqual(self.exrv.unsaved, True)
            self.save_run()

    #----------------------------------------------------------
    # copy the original data file:
    #----------------------------------------------------------
    def copy_data_files(self, data_file_orig):
        '''Copy the data files with ending .DAT and .ASC
        in order to run '__test__.py' without changing the
        original data files or associated pickle files with
        the same name. The new data files have the same name
        and path as the original data file extended with "_test".
        '''
        # get original dir path
        dir_path = os.path.dirname(data_file_orig)
        file_name = os.path.basename(data_file_orig)
        file_split = file_name.split('.')
        # copy the "*.ASC"-file
        file_name_orig_asc = os.path.join(dir_path, file_split[0] + '.ASC')
        file_name_test_asc = os.path.join(dir_path, file_split[0] + '_test.ASC')
        copy(file_name_orig_asc, file_name_test_asc)
        # copy the "*.DAT"-file
        file_name_orig_dat = os.path.join(dir_path, file_split[0] + '.DAT')
        file_name_test_dat = os.path.join(dir_path, file_split[0] + '_test.DAT')
        copy(file_name_orig_dat, file_name_test_dat)
        # return the path to the new "*_test.DAT"-file
        return file_name_test_dat

    #----------------------------------------------------------
    # delete the test data files after the test has been run:
    #----------------------------------------------------------
    def delete_test_data_files(self, data_file):
        print('XXX Delete the auxilary test data files if they exist --------------------')

        dir_path = os.path.dirname(data_file)
        file_name = os.path.basename(data_file)
        file_split = file_name.split('.')

        # delete the "*.ASC"-file
        file_name_orig_asc = os.path.join(dir_path, file_split[0] + '.ASC')
        file_name_test_asc = os.path.join(dir_path, file_split[0] + '_test.ASC')
        if os.path.exists(file_name_test_asc):
            print('--- test data file asc removed: ', file_name_test_asc, ' ---')
            os.remove(file_name_test_asc)
        else:
            print('--- test data file asc does not exist: ', file_name_test_asc, ' ---')
        # delete the "*.DAT"-file
        file_name_orig_dat = os.path.join(dir_path, file_split[0] + '.DAT')
        file_name_test_dat = os.path.join(dir_path, file_split[0] + '_test.DAT')
        if os.path.exists(file_name_test_dat):
            print('--- test data file dat removed: ', file_name_test_dat, ' ---')
            os.remove(file_name_test_dat)
        else:
            print('--- test data file dat does not exist: ', file_name_test_dat, ' ---')

    #----------------------------------------------------------
    # delete the pickle file if it exists:
    #----------------------------------------------------------
    def delete_pickle_file(self, data_file):
        print('XXX Delete the pickle file if it exists--------------------')

        dir_path = os.path.dirname(data_file)
        file_name = os.path.basename(data_file)
        file_split = file_name.split('.')
        pickle_file_name = os.path.join(dir_path, file_split[0] + '.pickle')
        if os.path.exists(pickle_file_name):
            print('--- pickle file removed: ', pickle_file_name, ' ---')
            os.remove(pickle_file_name)
        else:
            print('--- pickle file does not exist: ', pickle_file_name, ' ---')

    #----------------------------------------------------------
    # construct ExRunView and show attributes
    #----------------------------------------------------------
    def construct_exrv(self, data_file):
        print('XXX Construct ExRunView --------------------')
        self.exrv = ExRunView(data_file=data_file)
        print('--- GET: data_file = ', self.exrv.data_file, '---')
        print('--- GET: unsaved = ', self.exrv.unsaved, '---')

    #----------------------------------------------------------
    # get attributes of ExRunView
    #----------------------------------------------------------

    def get_attr(self, name):
        if name == 'age':
            return self.get_age()
        elif name == 'cm_key':
            return self.get_cm_key()
        elif name == 'flu_list':
            return self.get_flu_list()
        elif name == 's_tex_z':
            return self.get_s_tex_z()

    def get_age(self):
        print('XXX Get attributes of ExRunView --------------------')
        age = self.exrv.model.ex_type.age
        print('--- GET: age (old) = ', age , '---')
        return age

    def get_cm_key(self):
        print('XXX Get attributes of ExRunView --------------------')
        cm_key = self.exrv.model.ex_type.ccs.concrete_mixture_key
        print('--- GET: ccs.cm_key (old) = ', cm_key , '---')
        return cm_key

    def get_flu_list(self):
        print('XXX Get attributes of ExRunView --------------------')
        flu_list = self.exrv.model.ex_type.ccs.fabric_layup_list
        print('--- GET: ccs.flu_list (old)= ', flu_list , '---')
        return flu_list

    def get_s_tex_z(self):
        print('XXX Get attributes of ExRunView --------------------')
        s_tex_z = self.exrv.model.ex_type.ccs.fabric_layup_list[0].s_tex_z
        print('--- GET: ccs.flu_list[0].s_tex_z (old)= ', s_tex_z, '---')
        return s_tex_z

    #----------------------------------------------------------
    # save settings in pickle file
    #----------------------------------------------------------
    def save_run(self):
        print('XXX save settings in pickle file --------------------')
        self.exrv.save_run()
        print('--- SAVE RUN ---')
        print('--- pickle file saved to:', self.exrv.model.pickle_file_name, ' ---')
        print('--- GET: unsaved = ', self.exrv.unsaved, '---')

    #----------------------------------------------------------
    # change attribute of ExRunView
    #----------------------------------------------------------

    def change_attr(self, name):
        if name == 'age':
            return self.change_age()
        elif name == 'cm_key':
            return self.change_cm_key()
        elif name == 'flu_list':
            return self.change_flu_list()
        elif name == 's_tex_z':
            return self.change_s_tex_z()

    def change_age(self):
        print('XXX Change age --------------------')
        age_old = self.exrv.model.ex_type.age
        self.exrv.model.ex_type.age += 1
        print('--- SET: age (new)= ', self.exrv.model.ex_type.age , '---')
        print('--- GET: unsaved = ', self.exrv.unsaved, '---')
        return self.exrv.model.ex_type.age

    def change_cm_key(self):
        print('XXX Change concrete matrix in ccs --------------------')
        cm_key_old = self.exrv.model.ex_type.ccs.concrete_mixture_key
        if cm_key_old == 'FIL-10-09':
            self.exrv.model.ex_type.ccs.concrete_mixture_key = 'PZ-0708-1'
        elif cm_key_old == 'PZ-0708-1':
            self.exrv.model.ex_type.ccs.concrete_mixture_key = 'FIL-10-09'
        cm_key_new = self.exrv.model.ex_type.ccs.concrete_mixture_key
        print('--- SET: ccs.cm_key (new)= ', cm_key_new , '---')
        print('--- GET: unsaved = ', self.exrv.unsaved, '---')
        return cm_key_new

    def change_flu_list(self):
        print('XXX Append ccs.flu_list --------------------')
        self.exrv.model.ex_type.ccs.fabric_layup_list.append(plain_concrete(0.01))
        flu_list_new = self.exrv.model.ex_type.ccs.fabric_layup_list
        print('--- SET: ccs.fabric_layup_list (new)= ', flu_list_new , '---')
        print('--- GET: unsaved = ', self.exrv.unsaved, '---')
        return flu_list_new

    def change_s_tex_z(self):
        print('XXX Change s_tex_z in ccs.flu[0].s_tex_z --------------------')
        s_tex_z = self.exrv.model.ex_type.ccs.fabric_layup_list[0].s_tex_z
        self.exrv.model.ex_type.ccs.fabric_layup_list[0].s_tex_z += 1.0
        s_tex_z_new = self.exrv.model.ex_type.ccs.fabric_layup_list[0].s_tex_z
        print('--- SET: ccs.flu[0].s_tex_z (new)= ', s_tex_z_new , '---')
        print('--- GET: unsaved = ', self.exrv.unsaved, '---')
        return s_tex_z_new

    #----------------------------------------------------------
    # change data file to another file and then change it back
    #----------------------------------------------------------
    def reload_data_file(self, data_file_old, data_file_new):
        print('XXX Switch data file to another file and then back again --------------------')
        self.exrv.data_file = data_file_new
        print('--- SET: data file (new): ', self.exrv.data_file)
        if self.exrv.unsaved == True:
            self.exrv.save_run()
            print('--- SAVE RUN (new) ---')
        self.exrv.data_file = data_file_old
        print('--- SET: data file (back to old): ', self.exrv.data_file)


    def tearDown(self):
        print('\n')
        print('#------------------------------------------')
        print('# tearDown')
        print('#------------------------------------------')
        self.delete_test_data_files(self.data_file_orig_1)
        self.delete_test_data_files(self.data_file_orig_2)


if __name__ == "__main__":
    unittest.main()





import traits.api as tr
from bmcs_beam.beam_config.boundary_conditions import BoundaryConditions, BoundaryConfig
from bmcs_cross_section.cs_design import CrossSectionDesign
from bmcs_utils.api import InteractiveModel, \
    Int, Item, View, Float, Range, Button, ButtonEditor, mpl_align_yaxis_to_zero, \
    mpl_show_one_legend_for_twin_axes, FloatRangeEditor
from sympy.physics.continuum_mechanics.beam import Beam
import sympy as sp
import numpy as np
from numbers import Number

class BeamDesign(CrossSectionDesign):

    name = 'Beam Design'

    n_x = Int(100)
    L = Float(5000)
    F = Float(-1000)
    # TODO [HS]: make the conf a dropdown menu when the implementation is done by [RC]
    beam_conf_name = BoundaryConfig.THREE_PB
    beam_conf_name_slider = Range(0.)

    # beam_configs = BoundaryConfig
    # beam_conf = Enum(values='')

    beam = tr.Instance(Beam)

    tree = []

    def _beam_default(self):
        return BoundaryConditions.get_configured_beam(self.L, self.F, self.beam_conf_name)

    x = tr.Property(depends_on='L, n_x')

    @tr.cached_property
    def _get_x(self):
        return np.linspace(0, self.L, self.n_x)

    @tr.observe("beam_conf_name_slider")
    def notify_slider_change(self, event):
        self.beam_conf_name = BoundaryConfig(int(event.new))
        self.beam = BoundaryConditions.get_configured_beam(self.L, self.F, self.beam_conf_name)

    @tr.observe("L")
    def notify_L_change(self, event):
        self.beam = BoundaryConditions.get_configured_beam(event.new, self.F, self.beam_conf_name)

    @tr.observe("F")
    def notify_F_change(self, event):
        self.beam = BoundaryConditions.get_configured_beam(self.L, event.new, self.beam_conf_name)

    # add_force_btn = Button()
    #
    # def _add_force_btn_fired(self):
    #     self.L += 100

    ipw_view = View(
        Item('L', latex='L \mathrm{[mm]}'),
        Item('F', latex='F \mathrm{[N]}'),
        Item('n_x', latex='n_x'),
        Item('beam_conf_name_slider', editor=FloatRangeEditor(label='Beam config', n_steps=5, low=0, high=5)),
        # Item('beam_conf', editor=EnumEditor(label='Beam config', options_tuple_list=beam_configs)),
        # Item('add_force_btn', editor=ButtonEditor(label='Force', icon='plus'))
    )

    def solve_beam_for_reactions(self):
        # TODO: fix this
        #  The following is a quick fix in order for 4pb to work, without it the self.beam is somehow corrupted after
        #  using the function get_Fw() in DeflectionProfile so it's not able to solve the beam for 4pb!
        self.beam = BoundaryConditions.get_configured_beam(self.L, self.F, self.beam_conf_name)

        reactions_names = []

        for load in self.beam.applied_loads:
            load_value = load[0]
            if not isinstance(load_value, Number):
                reactions_names.append(load_value)

        self.beam.solve_for_reaction_loads(*reactions_names)

    def get_M_x(self, solve_beam_first=True):
        if solve_beam_first:
            self.solve_beam_for_reactions()
        x = sp.symbols('x')
        M_ = self.beam.bending_moment().rewrite(sp.Piecewise)
        get_M = sp.lambdify(x, M_, 'numpy')
        return get_M(self.x)

    def get_Q_x(self, solve_beam_first=True):
        if solve_beam_first:
            self.solve_beam_for_reactions()
        x = sp.symbols('x')
        Q_ = self.beam.shear_force().rewrite(sp.Piecewise)
        get_Q = sp.lambdify(x, Q_, 'numpy')
        Q_x = get_Q(self.x)
        return Q_x

    def plot_MQ(self, ax2, ax3):
        x = self.x
        M_scale = 1e+6
        Q_scale = 1000

        M_x = self.get_M_x(solve_beam_first=True) / M_scale
        Q_x = self.get_Q_x(solve_beam_first=False) / Q_scale

        ax2.plot(x, M_x, color='red', label='moment [kNm]')
        ax2.fill_between(x, 0, M_x, color='red', alpha=0.1)
        ax2.set_ylabel('M [kNm]')
        ax2.invert_yaxis()

        ax3.plot(x, Q_x, lw=0.1, color='green', label='shear [kN]')
        ax3.fill_between(x, Q_x, 0, color='green', alpha=0.15)
        ax3.set_ylabel('Q [kN]')
        ax3.set_xlabel('x [mm]')
        ax3.invert_yaxis()
        mpl_show_one_legend_for_twin_axes(ax2, ax3)
        mpl_align_yaxis_to_zero(ax2, ax3)

    def subplots(self, fig):
        ax1, ax2 = fig.subplots(2, 1)
        ax3 = ax2.twinx()
        return ax1, ax2, ax3

    # @todo: [HS] this limits the beam design object to just a MQ profile
    # it is, however meant to be only useful for the deflection calculation.
    # For the shear zone - the interface should be generalized and the
    # beam design should provide more services, namely the access to
    # all the components of the beam, draw the plan from the side view,
    # cross sectional view - indeed the whole design including the supports
    # and loading.
    #
    # An alternative is to declare this package just to a beam_statical_system
    # which is purely 1D and not to call it design. Then a BeamDesign would
    # contain both - beam_bc (a statical system) and cs_design. It would provide
    # functionality to change the parameters of the design - including the
    # length, load configuration and supports (BC in one word).
    #
    def update_plot(self, axes):
        ax1, ax2, ax3 = axes
        BoundaryConditions.plot(ax1, self.beam)
        self.plot_MQ(ax2, ax3)
        ax1.axis('equal')
        ax1.autoscale(tight=True)

    # Quick fix: needed for [bmcs_shear_zone]
    def plot_reinforcement(self, ax):
        L = self.L
        for z in self.cross_section_layout.reinforcement.z_j:
            ax.plot([0,L],[z,z], lw=2, color='brown')


'''
Created on 2. 2. 2014

@author: Vancikv

Example of cross section assembly - Rectangular concrete
 cross section with uniformly distributed textile layers
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCTexUniform

from matplotlib.figure import \
    Figure

from matplotlib.backends.backend_agg import \
    FigureCanvasAgg

from bmcs_beam.mxn.utils import \
    get_outfile

uni_layers = RLCTexUniform(n_layers=10,
                           material='default_fabric',
                           material_law='fbm')
'''Uniformly distributed textile layers
'''

ge = MCSGeoRect(height=0.4, width=0.3)
'''Cross section geometry
'''

mcs = MatrixCrossSection(geo=ge,
                         n_cj=20,
                         material='default_mixture',
                         material_law='constant')
'''Matrix cross section
'''

cs = CrossSection(reinf=[uni_layers],
                  matrix_cs=mcs,
                  eps_lo=0.002,
                  eps_up=-0.0033,
                  )

print('normal force', cs.N)
print('moment', cs.M)

fig = Figure(figsize=(10, 7), dpi=80, facecolor='white')
canvas = FigureCanvasAgg(fig)
ax = fig.add_subplot(1, 1, 1)
cs.plot_geometry(ax)

canvas.print_figure(get_outfile(folder_name='.mxn',
                                file_name='ex03.png'))




from mayavi.core.source import Source
from numpy import zeros, mgrid, c_, indices, transpose, array, arange, \
    asarray, ix_, ones, random, floor, outer, diagflat, vdot, minimum
from traits.api import \
    HasTraits, Float, Int, Array, Property, cached_property, \
    Tuple, List, Str, on_trait_change, Button, Delegate, \
    Instance, Trait


from mathkit.geo.geo_ndgrid import GeoNDGrid, GridPoint
from functools import reduce


# tvtk related imports
#
class MFnNDGrid(GeoNDGrid):
    '''
    Grid extended with functional interface to set the point data explicitly.

    The point data can be set using the method set_values_in_range 
    and extracted using the get_value method for an arbitrary position 
    within the domain.  
    '''
    point_values = Array(float)

    def _point_values_default(self):
        return []

    @on_trait_change('shape,active_dims')
    def _reset_point_values(self):
        self.point_values = ones(self.n_act_nodes, dtype=float)

    def set_value_all(self, value):
        ''' Set all entries to the specified value.'''
        self.point_values[:] = value

    def _get_idx(self, x):
        ''' Get index for a specified x,y coordinate '''

        n_act_elems = array(self.n_act_nodes[:], int)
        n_act_elems[self.dim_indices] -= 1

#        idx = array( floor( (( array(x, float)-array(self.x_mins[:], float)) /
#                             ( array(self.x_maxs[:], float) -
#                               array(self.x_mins[:], float) ) )
#                               / n_act_elems ), int )

        d_coord = array(self.x_maxs[:], float) - array(self.x_mins[:], float)
        i = 0
        for j in d_coord:
            if d_coord[i] == 0.:
                d_coord[i] = 1.
            i += 1

        x_coord = array(x, float) - array(self.x_mins[:], float)
        idx = array(floor(x_coord / d_coord * n_act_elems), int)

        return idx

    def set_values_in_box(self, value, x_min, x_max, min_max=0):
        ''' Set entries within the specified box to the specified value '''
        idx_mins = self._get_idx(x_min)
        idx_mins[self.dim_indices] += 1
        idx_maxs = self._get_idx(x_max) + 1
        slices = [slice(idx_min, idx_max)
                  for idx_min, idx_max in zip(idx_mins, idx_maxs)]
        self.point_values[slices] = value

    def get_value(self, x):
        '''Return interpolated value.
        '''
        # The interplation is done using bilinear shape functions.
        # The numpy array functionality is exploited to locate
        # the interpolation cell coordinates and the point values in
        # its corner nodes.

        # get the idx of the bottom left corner
        idx_mins = minimum(self._get_idx(x), array(self.shape, int) - 1)
        # idx of the upper right corner
        idx_maxs = idx_mins + 1
        idx_maxs[self.dim_indices] += 1
        # construct the slice for the cell
        slices = array(idx_mins, dtype='object_')
        slices[self.dim_indices] = array([slice(idx_mins[i], idx_maxs[i])
                                          for i in self.dim_indices])
        # get the corner cell values (note, this operation
        # includes dimensional reduction, i.e. corresponding
        # to the currently active indices
        cell_values = self.point_values[tuple(slices)]

        # get slice for the coordinates of the lower left corner
        lower_left_slices = (tuple(self.dim_indices), ) + tuple(idx_mins)
        # delta of the specified x with respect to lower left corner
        dx = (array(x, float)[self.dim_indices] - self.grid[lower_left_slices])

        # get the slice of the upper right corner
        idx_mins1 = idx_mins
        idx_mins1[self.dim_indices] += 1
        upper_right_slices = (tuple(self.dim_indices), ) + tuple(idx_mins1)
        lengths = self.grid[upper_right_slices] - self.grid[lower_left_slices]
        coeffs = dx / lengths[self.dim_indices]

        # perform the interpolation using broadcasting
        shapes = diagflat([1 for i in self.dim_indices]) + 1
        c_list = [array([1 - coeffs[i], coeffs[i]], float).
                  reshape(tuple(shapes[i])) for i in range(self.n_dims)]
        N_fn = reduce(lambda x, y: x * y, c_list)

        value = vdot(N_fn, cell_values)
        return value

    def __call__(self, active_x):
        '''
        Make the function callable as well. 
        '''
        x = zeros(3, float)
        x[self.dim_indices] = array(active_x, float)
        return self.get_value(list(x))

    def _get_scalars(self):
        return self.point_values.flatten()

if __name__ == '__main__':

    from mayavi.modules.api import Outline, Surface
    from mayavi.scripts import mayavi2
    mayavi2.standalone(globals())

    # Tests to write - set a quadrangle and values in the corner nodes
    # check the returned values for all the corner nodes
    #
    # Check the interpolation in four internal positions.

    # 'mayavi' is always defined on the interpreter.
    # mayavi.new_scene()
    # Make the data and add it to the pipeline.
    mfn = MFnNDGrid(active_dims=['x', 'y'], shape=(1, 1, 1),
                    x_maxs=GridPoint(x=5, y=5, z=15))
    mfn.set_values_in_box(4.3, [1, 1, 1], [5, 5, 2])
    print('value at (5,0,0)', mfn.get_value([5, 5, 0]))
    print('value at (1.1,1.1,1.1)', mfn.get_value([1.1, 1.1, 1]))
    print('value at (0.5,0.5)', mfn([0.5, 0.5]))
#     mayavi.add_source(mfn)
#     # Visualize the data.
#     mayavi.add_module(Outline())
#     mayavi.add_module(Surface())
    mfn.configure_traits()



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Created on May 25, 2009 by Rostislav Chudoba
#
#-------------------------------------------------------------------------------

import matplotlib
matplotlib.use('WXAgg')
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wx import NavigationToolbar2Wx

from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
from matplotlib.axes import Axes
from matplotlib.lines import Line2D
from matplotlib.figure import Figure
import wx
from numpy import *

from traitsui.api import View, Item
from traits.api import Any, Instance, on_trait_change
from etsproxy.traits.ui.wx.editor import Editor
from etsproxy.traits.ui.basic_editor_factory import BasicEditorFactory
from .mfn_plot_adapter import MFnMultiPlotAdapter
from matplotlib.ticker import MaxNLocator

class _MFnMatplotlibEditor(Editor):
    
    
    # @todo This is not used here, either activate or delete
    scrollable  = True
    
    # adjustable parameters
    adapter = Instance( MFnMultiPlotAdapter )
    
    # the plot representation in matplotlib
    figure = Instance(Figure(facecolor = MFnMultiPlotAdapter().bgcolor), ())

    def init(self, parent):
        
        factory = self.factory
        self.adapter = factory.adapter
        
        self.control = self._create_canvas(parent)
        self.value.on_trait_change( self.update_editor, 'data_changed' )

    def update_editor(self):
        figure = self.figure
        axes = figure.add_subplot(111)
        canvas = figure.canvas
        if canvas is None:
            pass
        else:
            figure.delaxes( axes )
            self._refresh_plot()
            figure.canvas.draw()

    def _create_canvas(self, parent):
        """ Create the MPL canvas. """
        # The panel lets us add additional controls.
        fig = self.figure
    
        panel = wx.Panel(parent, -1, style=wx.CLIP_CHILDREN)
        sizer = wx.BoxSizer(wx.VERTICAL)
        panel.SetSizer(sizer)
#
        # matplotlib commands to create a canvas
        mpl_control = FigureCanvas( panel, -1, fig )
        toolbar = NavigationToolbar2Wx(mpl_control)
        sizer.Add(toolbar, 0, wx.EXPAND)
        sizer.Add(mpl_control, 1, wx.LEFT | wx.TOP | wx.GROW )
        fig.canvas.SetMinSize((100,100))
        return panel

    def _refresh_plot( self ):

        a = self.adapter
        figure = self.figure
        mfn_multiline = self.value
        plot_set = []
        for data in mfn_multiline.xdata:
            plot_set.append(any(array(data != [0.,1.])))
        idx = where(array(plot_set) == True)[0]
        xdata = []
        ydata = []
        for index in idx:
            xdata.append(mfn_multiline.xdata[index]),
            ydata.append(mfn_multiline.ydata[index])
        legend = array(a.legend_labels)[idx]
        color = array(a.mline_color)[idx]
        style = array(a.mline_style)[idx]
        width = array(a.mline_width)[idx]
        
        if a.var_x != '':
            label_x = getattr( self.object, a.var_x )
        else:
            label_x = a.label_x
            
        if a.var_y  != '':
            label_y = getattr( self.object, a.var_y )
        else:
            label_y = a.label_y
            
        axes = figure.add_subplot(111)
        
        for x,y,c,s,w in zip(xdata[:], ydata[:], \
                         color[:], style[:], width[:]):
            axes.plot(x, y, color = c, linestyle = s, linewidth = w)
        axes.set_xlabel( a.label_x, weight = 'semibold')
        axes.set_ylabel( a.label_y, weight = 'semibold')
        axes.set_title( a.title,\
                        size = 'large', color = 'black',\
                        weight = 'bold', position = (.5,1.03))
        axes.set_axis_bgcolor(color = 'white')
        axes.ticklabel_format(scilimits = a.scilimits)
        axes.grid(color='gray', linestyle='--', linewidth=0.1, alpha = 0.4)
        axes.legend((legend), loc = 'best' )
        axes.set_xscale(a.xscale)
        axes.set_yscale(a.yscale)
        if a.xticks == 0:
            pass
        else:
            axes.xaxis.set_major_locator(MaxNLocator(a.xticks))
        if a.yticks == 0:
            pass
        else:
            axes.yaxis.set_major_locator(MaxNLocator(a.yticks))


class MFnMatplotlibEditor(BasicEditorFactory):

    klass = _MFnMatplotlibEditor
    
    adapter = Instance( MFnMultiPlotAdapter )
    def _adapter_default(self):
        return MFnMultiPlotAdapter()
    


'''
Created on Apr 9, 2010

@author: alexander
'''

import os

from matresdev.db.simdb.simdb import simdb
import numpy as np


def dot2comma(value):
    value = value.replace(',', '.')
    return float(value)


file_name = os.path.join(simdb.exdata_dir, 'bending_tests',
                         'ZiE_2011-06-08_BT-12c-6cm-0-TU', 'BT-12c-6cm-0-Tu-V4.raw')

# file contains both loading- and unloading path:
#
#file_name = '/home/alexander/simdb/exdata/bending_tests/ZiE_2011-06-08_BT-12c-6cm-0-TU/BT-12c-6cm-0-Tu-V2-converted.csv'


def loadtxt_bending(file_name):
    '''Return an data array of the bending test
    - first column: displacement [mm]
    - second column: compression strains at midfield [%]
    - third column: load [N]
    '''
    try:
        # Return an data array for the loading path (1 block).
        # load raw-data in case of loading path only
        # (no additional unloading path recorded below the first data block in the file)
        # in this case loadtxt works properly'''
        data_arr = np.loadtxt(file_name,
                              delimiter=';',
                              skiprows=41,
                              converters={
                                  1: dot2comma, 2: dot2comma, 3: dot2comma},
                              usecols=[1, 2, 3])
        print('loadtxt_bending: data_arr contains only loading path')

    except IndexError:
        print('loadtxt_bending: data_arr contains loading- and unloading path')
        data_arr = loadtxt_2blocks(file_name)

    return data_arr


def loadtxt_2blocks(file_name):
    '''Return an data array consisting of the loading AND unloading path (merge 2 blocks in the data file).
    in this case loadtxt doesn't work as the data file consits of 2 blocks'''
    file_ = open(file_name, 'r')
    lines = file_.readlines()

    data_arr_1 = np.zeros(3)
    data_arr_2 = np.zeros(3)

    start_n_blocks = []
    end_n_blocks = []

    # determine the starting number and end number of the data blocks 1 and 2:
    #
    n = 0
    for line in lines:
        line_split = line.split(';')
        if line_split[0] == '"Probe"':
            # first block normally starts with line 43
            # the starting line of the second block needs to be determined
            # 27 lines after the keyword "Probe" the data is recorded in both blocks
            #
            start_n_blocks.append(n + 28)
        if line_split[0] == '"Probe"':
            end_n_blocks.append(n)
        n += 1

    if len(end_n_blocks) != 1:
        # add the line number of the last line
        # this corresponds to the last line of block 2 if it is recorded
        #
        end_n_blocks.append(len(lines))
        end_n_blocks = end_n_blocks[1:]

#    print 'start_n_blocks', start_n_blocks
#    print 'end_n_blocks', end_n_blocks

    # convert data to array for blocks 1:
    #
    for line in lines[start_n_blocks[0]:end_n_blocks[0]]:
        line_split = line.split(';')
        line_arr = np.array([dot2comma(line_split[1]),
                             dot2comma(line_split[2]),
                             dot2comma(line_split[3])],
                            dtype=float)
        data_arr_1 = np.vstack([data_arr_1, line_arr])

    # convert data to array for blocks 2:
    #
    for line in lines[start_n_blocks[1]:end_n_blocks[1]]:
        line_split = line.split(';')
        line_arr = np.array([dot2comma(line_split[1]),
                             dot2comma(line_split[2]),
                             dot2comma(line_split[3])],
                            dtype=float)
        data_arr_2 = np.vstack([data_arr_2, line_arr])

    # remove line with zeros
    #
    data_arr = np.vstack([data_arr_1[1:], data_arr_2[1:]])
    return data_arr

if __name__ == '__main__':

    data_arr = loadtxt_bending(file_name)
    print('data_arr', data_arr)



#-------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Jan 19, 2010 by: rch

from matplotlib.figure import \
    Figure
import os
from pyface.api import \
    ImageResource, confirm, YES
from traits.api import \
    Instance, File, List, on_trait_change, Bool, \
    Event, Enum
from traitsui.api import \
    View, Item, ModelView, HSplit, VSplit, \
    Group, FileEditor
from traitsui.key_bindings import \
    KeyBinding, KeyBindings
from traitsui.menu import \
    Action, HelpAction, Menu, \
    MenuBar, ToolBar

from .ex_run import \
    ExRun
from matresdev.db.simdb.simdb import simdb
from util.traits.editors.mpl_figure_editor import \
    MPLFigureEditor


class ExRunView(ModelView):

    #-------------------------------------------------------------------------
    # Initialization
    #-------------------------------------------------------------------------

    filter = List()

    def _filter_default(self):
        return ['*.DAT;*.raw']

    # Overload the constructor to load the initial experiment.
    # The model of a ModelView cannot be None.
    #
    def __init__(self, **kw):
        super(ExRunView, self).__init__(**kw)
        self.load_run()
        self._reset_plot_template_list()
        self.redraw()

    # Register the ui window in order to be able to change its state
    #
    def init(self, ui_info):
        self._ui_info = ui_info

    #-------------------------------------------------------------------------
    # Model manipulation
    #-------------------------------------------------------------------------

    # Current experiment run
    #
    data_file = File

    def _data_file_default(self):
        # get the default form of the model
        #        ex_path = os.path.join( simdb.exdata_dir,
        #                                'tensile_tests',
        #                                'TT-9a',
        #                                'TT09-9a-V1.DAT' )
        ex_path = os.path.join(simdb.exdata_dir,
                               'plate_tests',
                               'PT-10a',
                               'PT11-10a.DAT')

        return ex_path

    # Reset the ex_run instance when the file was changed
    #
    @on_trait_change('data_file')
    def _reset_model(self, obj, name, old, new):
        print('RESET MODEL')
        model = self.model
        if model:
            # check if the model was changed and ask if it is to be saved
            #
            if model.unsaved:

                answer = confirm(self._ui_info.ui.control, 'Run changed, save it?',
                                 title='Save confirmation',
                                 cancel=False,
                                 default=YES)

                if answer == YES:
                    # ask whether the modified run should be saved
                    #
                    self.save_run()
                self.model.unsaved = False

        self.load_run()
        self._reset_plot_template_list()
        self.redraw()

    # The ex_run instance itself
    #
    model = Instance(ExRun)

    #-------------------------------------------------------------------------
    # Drawing
    #-------------------------------------------------------------------------

    # The variable controlling the redraw button. It is set True whenever
    # an input value of the experiment run was changed.
    #
    def set_changed(self):
        '''Callback to be registered with the model. It gets called whenever
        the change_event event of the ex_run has been triggered.
        '''
        self.redraw()

    unsaved = Bool(False)

    def listen_to_unsaved(self):
        self.unsaved = self.model.unsaved

    @on_trait_change('model')
    def _reset_model_listeners(self, obj, name, old_model, new_model):
        '''Delete the listeners of the old object reacting to the redraw tag
        of the experiment run. Bind the redraw listener to the newly
        attached model.
        '''
        if old_model:
            old_model.on_trait_change(
                self.set_changed, 'change_event', remove=True)
            old_model.on_trait_change(
                self.listen_to_unsaved, 'unsaved', remove=True)
        if new_model:
            new_model.on_trait_change(self.set_changed, 'change_event')
            new_model.on_trait_change(self.listen_to_unsaved, 'unsaved')

    @on_trait_change('plot_template')
    def redraw(self, ui_info=None):
        ''' Use the currently selected plot template to plot it in the Figure.
        '''
        print('*** replotting ***')

        # map the array dimensions to the plot axes
        #
        figure = self.figure
        axes = figure.gca()
        axes.clear()

        proc_name = self.model.ex_type.plot_templates[self.plot_template]
        plot_processor = getattr(self.model.ex_type, proc_name)
        plot_processor(axes)

        self.data_changed = True

    # -------------------------------------------------------------------------
    # Persistence management
    # -------------------------------------------------------------------------

    def load_run(self, ui_info=None):
        '''Read the run from the current data file into the model.
        '''
        self.model = ExRun(self.data_file)
        self.listen_to_unsaved()

    def save_run(self, ui_info=None):
        '''Save the model into associated pickle file.
        The source DAT file is unaffected.
        '''
        self.model.save_pickle()
        self.model.unsaved = False

    def reset_run(self, ui_info=None):
        '''Save the model into associated pickle file.
        The source DAT file is unaffected.
        '''
        answer = confirm(self._ui_info.ui.control, 'Really reset? Changes will be lost!',
                         title='Reset confirmation',
                         cancel=False,
                         default=YES)

        if answer == YES:
            # ask whether the modified run should be saved
            #
            self.load_run(ui_info)
            self.redraw()
            self.model.unsaved = False

    #-------------------------------------------------------------------
    # PLOT OBJECT
    #-------------------------------------------------------------------

    figure = Instance(Figure)

    def _figure_default(self):
        figure = Figure(facecolor='white')
        figure.add_axes([0.08, 0.13, 0.85, 0.74])
        return figure

    # event to trigger the replotting - used by the figure editor
    #
    data_changed = Event

    # selected plot template
    #
    plot_template = Enum(values='plot_template_list')

    def _plot_template_default(self):
        return self.model.ex_type.default_plot_template

    # list of availble plot templates
    # (gets extracted from the model whenever it's been changed)
    #
    plot_template_list = List

    def _reset_plot_template_list(self):
        '''Change the selection list of plot templates.

        This method is called upon every change of the model. This makes the viewing of
        different experiment types possible.
        '''
        self.plot_template_list = list(self.model.ex_type.plot_templates.keys())

    #-------------------------------------------------------------------------
    # UI specification
    #-------------------------------------------------------------------------

    key_bindings = Instance(KeyBindings)

    def _key_bindings_default(self):
        """ Trait initializer. """

        key_bindings = KeyBindings(
            KeyBinding(
                binding1='Ctrl-s',
                description='Save the run',
                method_name='save_run'
            ),

            KeyBinding(
                binding1='Ctrl-r',
                description='Plot the response',
                method_name='reset_run'
            )
        )

        return key_bindings

    def default_toolbar(self):
        return ToolBar(
            Action(name="Save",
                   tooltip='Save run',
                   enabled_when='unsaved',
                   image=ImageResource('save'),
                   action="save_run"),
            Action(name="Reset",
                   tooltip='Reset run',
                   enabled_when='unsaved',
                   image=ImageResource('reset'),
                   action="reset_run"),
            image_size=(22, 22),
            show_tool_names=False,
            show_divider=True,
            name='exrun_toolbar')

    def default_menubar(self):
        return MenuBar(Menu(Action(name="&Open",
                                   action="load_run"),
                            Action(name="&Save",
                                   action="save_run"),
                            Action(name="&Exit",
                                   action="exit"),
                            name="&File"),
                       Menu(Action(name="About PStudy",
                                   action="about_pstudy"),
                            HelpAction,
                            name="Help")
                       )

    def default_traits_view(self):

        return View(HSplit(
            VSplit(
                Item('data_file@', editor=FileEditor(filter_name='filter'),
                     show_label=False),
                Group(
                    Item('figure', editor=MPLFigureEditor(),
                         resizable=True, show_label=False),
                    id='simexdb.plot_sheet',
                    label='plot sheet',
                    dock='tab',
                ),
                Group(
                    Item('plot_template'),
                    columns=1,
                    label='plot parameters',
                    id='simexdb.plot_params',
                    dock='tab',
                ),
                id='simexdb.plot.vsplit',
                dock='tab',
            ),
            VSplit(
                Item('model@',
                     id='simexdb.run.split',
                     dock='tab',
                     resizable=True,
                     label='experiment run',
                     show_label=False),
                id='simexdb.mode_plot_data.vsplit',
                dock='tab',
                scrollable=True
            ),
            id='simexdb.hsplit',
            dock='tab',
        ),
            key_bindings=self.key_bindings,
            menubar=self.default_menubar(),
            toolbar=self.default_toolbar(),
            resizable=True,
            title='Simvisage: experiment database browser',
            id='simexdb',
            dock='tab',
            buttons=['Ok'],  # , 'Cancel' ],
            height=1.0,
            width=1.0)

if __name__ == '__main__':

    #    ex_path = os.path.join( simdb.exdata_dir, 'bending_tests', 'ZiE_2011-06-08_BT-12c-6cm-0-TU', 'BT-12c-6cm-0-Tu-V4.raw' )

    ex_path = os.path.join(simdb.exdata_dir, 'tensile_tests', 'dog_bone', '2011-06-10_TT-12c-6cm-90-TU_ZiE',
                           'TT-12c-6cm-90-TU-V1.DAT')

#    ex_path = os.path.join( simdb.exdata_dir, 'plate_tests', 'PT-6a-ibac',
#                            'PTi-6a-woSF', 'PTi-6a-woSF-V1.DAT' )

#    ex_path = os.path.join( simdb.exdata_dir, 'plate_tests', 'PT-10a',
#                            'PT10-10a.DAT' )

    doe_reader = ExRunView(data_file=ex_path)
    doe_reader.configure_traits()



'''
Created on Jan 17, 2014

@author: rch
'''

from traits.etsconfig.api import ETSConfig
ETSConfig.toolkit = 'qt4'

from .cross_section import \
    CrossSection
from .cross_section_component import \
    CrossSectionComponent
from .cross_section_state import \
    CrossSectionState
from .ecb_calib import \
    ECBCalib
from .matrix_laws import \
    MatrixLawBase
from .mxn_diagram import \
    MxNDiagram
from .mxn_tree_node import \
    MxNTreeNode
from .reinf_laws import \
    ReinfLawBase, ReinfLawFBM
from .view import \
    MxNTreeView



'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property

import numpy as np

from .reinf_law_base import \
    ReinfLawBase

class ReinfLawSteel(ReinfLawBase):
    f_yk = Float(500., enter_set=True, auto_set=False, input=True)
    E_s = Float(200000., enter_set=True, auto_set=False, input=True)
    eps_u = Float(0.025, enter_set=True, auto_set=False, input=True)

    eps_y = Property(Float, depends_on='+input')
    @cached_property
    def _get_eps_y(self):
        return self.f_yk / self.E_s

    eps_arr = Property(depends_on='+input')
    @cached_property
    def _get_eps_arr(self):
        return np.array([-self.eps_u, -self.eps_y, 0., self.eps_y, self.eps_u])

    sig_arr = Property(depends_on='+input')
    @cached_property
    def _get_sig_arr(self):
        return np.array([-self.f_yk, -self.f_yk, 0., self.f_yk, self.f_yk])

ReinfLawBase.db.constants['steel-default'] = ReinfLawSteel()



'''
Created on 9. 7. 2014

@author: Vancikv

This module contains tree nodes defined for specific classes
as oposed to the two generic node types defined in mxn_tree_view
that are sufficient for displaying the tree structure but
don't provide different classes with specific node behaviour.
'''

import os
import pickle

from traits.etsconfig.api import ETSConfig
from traitsui.api import \
    TreeNode
from traitsui.menu import \
    Menu

from bmcs_beam.mxn.cross_section import \
    CrossSection
from bmcs_beam.mxn.ecb_calib import \
    ECBCalib
from bmcs_beam.mxn.material_types import \
    MaterialTypeBase
from bmcs_beam.mxn.mxn_class_extension import \
    MxNClassExt
from bmcs_beam.mxn.mxn_diagram import \
    MxNDiagram
from bmcs_beam.mxn.mxn_tree_node import \
    ReinfLayoutTreeNode
from bmcs_beam.mxn.reinf_layout import \
    RLCTexLayer, RLCTexUniform, RLCBar
from bmcs_beam.mxn.use_cases import \
    UCDatabase, UCPStudyElementMxN, UCParametricStudy, \
    UseCaseContainer
from bmcs_beam.mxn.utils import \
    get_outfile
from .mxn_tree_view_handler import \
    plot_self, new_material, del_material


if ETSConfig.toolkit == 'wx':
    from traitsui.wx.tree_editor import \
        NewAction, DeleteAction, CopyAction, PasteAction
if ETSConfig.toolkit == 'qt4':
    from traitsui.qt4.tree_editor import \
        NewAction, DeleteAction, CopyAction, PasteAction
else:
    raise ImportError("tree actions for %s toolkit not availabe" % \
        ETSConfig.toolkit)


# =========================================================================
# Special TreeNode classes
# =========================================================================


class UCCTreeNode(TreeNode):

    def append_child(self, object_, child):
        """ Overriding this method ensures that upon dropping an
        object merely its copy is appended. Necessary to avoid
        multiple references to the same object in tree structure
        that would occur upon drag-n-dropping a child of
        SingleChildTreeNode onto the use case container.
        """
        copy_file = get_outfile(folder_name='.mxn',
                                file_name='temp.pkl')
        file_ = open(copy_file, 'wb')
        pickle.dump(child, file_, 1)
        file_.close()
        file_ = open(copy_file, 'rb')
        new_child = pickle.load(file_)
        file_.close()
        os.remove(copy_file)

        getattr(object_, self.children).append(new_child)


class SingleChildTreeNode(TreeNode):

    def append_child(self, object_, child):
        """ Overriding this method ensures that there is
        always only one child and that upon dropping an object
        merely its copy is assigned.
        """
        copy_file = get_outfile(folder_name='.mxn',
                                file_name='temp.pkl')
        file_ = open(copy_file, 'wb')
        pickle.dump(child, file_, 1)
        file_.close()
        file_ = open(copy_file, 'rb')
        new_child = pickle.load(file_)
        file.close()
        os.remove(copy_file)

        setattr(object, self.children, [new_child])

    def delete_child(self, object_, index):
        """ Overriding the delete method in order to ensure
        that a dragged object does not remove itself from its
        original location, i.e. the method does nothing because
        there should always be one child.
        """
        pass

# =========================================================================
# Use cases and enveloping nodes
# =========================================================================

use_case_container_node = UCCTreeNode(
    node_for=[UseCaseContainer],
    auto_open=True,
    children='tree_node_list',
    label='node_name',
    view='tree_view',
    menu=Menu(NewAction, PasteAction),
    add=[MxNDiagram, UCParametricStudy,
         ECBCalib],
)

parametric_study_node = TreeNode(
    node_for=[UCParametricStudy],
    auto_open=True,
    children='tree_node_list',
    label='node_name',
    view='tree_view',
    menu=Menu(plot_self,
              DeleteAction,
              ),
    add=[],
)

mxn_diagram_container_node = SingleChildTreeNode(
    node_for=[UCPStudyElementMxN],
    auto_open=True,
    children='tree_node_list',
    label='node_name',
    view='tree_view',
    menu=Menu(plot_self,
              PasteAction,
              DeleteAction),
    add=[MxNDiagram],
)

mxn_diagram_node = SingleChildTreeNode(node_for=[MxNDiagram],
                                       auto_open=True,
                                       children='tree_node_list',
                                       label='node_name',
                                       view='tree_view',
                                       add=[CrossSection],
                                       menu=Menu(plot_self,
                                                 CopyAction,
                                                 PasteAction,
                                                 DeleteAction),
                                       )

ecb_calib_node = SingleChildTreeNode(node_for=[ECBCalib],
                                     auto_open=True,
                                     children='tree_node_list',
                                     label='node_name',
                                     view='tree_view',
                                     add=[CrossSection],
                                     menu=Menu(plot_self,
                                               CopyAction,
                                               PasteAction,
                                               DeleteAction),
                                     )
# =========================================================================
# Database and subordinate nodes
# =========================================================================

''' Generic nodes are used for constitutive laws
'''

database_node = TreeNode(node_for=[UCDatabase],
                         auto_open=False,
                         children='tree_node_list',
                         label='node_name',
                         view='tree_view',
                         menu=Menu(DeleteAction),
                         )

database_subnode = TreeNode(node_for=[MxNClassExt],
                            auto_open=True,
                            children='tree_node_list',
                            label='node_name',
                            view='tree_view',
                            menu=Menu(new_material),
                            )

material_type_node = TreeNode(node_for=[MaterialTypeBase],
                              auto_open=False,
                              children='tree_node_list',
                              label='node_name',
                              view='tree_view',
                              menu=Menu(del_material),
                              )

# =========================================================================
# Cross section and subordinate nodes
# =========================================================================

''' Generic nodes are used for MatrixCrossSection and for constitutive laws
'''

cross_section_node = TreeNode(node_for=[CrossSection],
                              auto_open=True,
                              children='tree_node_list',
                              label='node_name',
                              view='tree_view',
                              menu=Menu(CopyAction, plot_self),
                              )

reinf_layout_node = TreeNode(node_for=[ReinfLayoutTreeNode],
                             auto_open=True,
                             children='tree_node_list',
                             label='node_name',
                             view='tree_view',
                             menu=Menu(plot_self, NewAction),
                             add=[RLCTexUniform,
                                   RLCTexLayer,
                                   RLCBar]
                             )

reinf_tex_uniform_node = TreeNode(node_for=[RLCTexUniform],
                                  auto_open=False,
                                  children='tree_node_list',
                                  label='node_name',
                                  view='tree_view',
                                  menu=Menu(plot_self, DeleteAction),
                                  )

reinf_tex_layer_node = TreeNode(node_for=[RLCTexLayer],
                                auto_open=False,
                                children='tree_node_list',
                                label='node_name',
                                view='tree_view',
                                menu=Menu(plot_self, DeleteAction),
                                )

reinf_bar_node = TreeNode(node_for=[RLCBar],
                          auto_open=False,
                          children='tree_node_list',
                          label='node_name',
                          view='tree_view',
                          menu=Menu(plot_self, DeleteAction),
                          )

# =========================================================================
# List of all custom nodes
# =========================================================================

custom_node_list = [mxn_diagram_container_node, mxn_diagram_node,
                    cross_section_node, material_type_node,
                    reinf_layout_node, reinf_tex_uniform_node,
                    reinf_tex_layer_node, reinf_bar_node,
                    database_node, database_subnode, ecb_calib_node,
                    use_case_container_node, parametric_study_node]



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Jan 19, 2010 by: rch

# @todo:
# - test case for levels_modified and for model_modified
# - other_factors should be explicitly listed - concept for level status
#   change 
# - legend for depth axis in the plots - done
# - show the output values in the input_table
#   in terms of DOE the input_table corresponds to the full_factorial design.
# - window menu (exit menu entry in the File menu)
# - threaded evaluation (estimation of computation time of a single run)
# - progress bar showing the number of simulations
# - purging of the factor space using regression / splines?
#   this would require the fractional factorial array
#
# Q: how to make a toolbar within a view without having to write a special editor?
# consider the tree on the left showing the studied space
#
import os
#os.environ['ETS_TOOLKIT'] = 'qt4'

#
from traits.api import \
    HasTraits, Float, Property, cached_property, \
    Instance, List, on_trait_change, Int, Tuple, Bool, \
    DelegatesTo, Event, Str, Button, Dict, Array, Any, \
    implements, File
    
from traitsui.api import \
    View, Item, Tabbed, VGroup, HGroup, ModelView, HSplit, VSplit, \
    CheckListEditor, EnumEditor, TableEditor, TabularEditor, Handler

from etsproxy.traits.ui.menu import Action, CloseAction, HelpAction, Menu, \
                                     MenuBar, NoButtons, Separator, ToolBar
                                         
from etsproxy.traits.ui.tabular_adapter \
    import TabularAdapter

from etsproxy.pyface.api import ImageResource, ProgressDialog
from .sim_todo import ToDo

from etsproxy.traits.ui.menu import OKButton
from .i_sim_model import ISimModel
from .i_sim_array import ISimArray
from .sim_array_view import SimArrayView

from etsproxy.traits.ui.file_dialog  \
    import open_file, save_file, FileInfo, TextInfo

import pickle
import os.path as path
    
from .sim_factor import \
    SimFactor, SimFloatFactor, SimIntFactor, SimEnumFactor
from .sim_output import SimOut

from numpy import array, linspace, frompyfunc, zeros, column_stack, \
                    log as ln, append, logspace, hstack, sign, trapz, mgrid, c_, \
                    zeros, arange, ix_

from math import exp, e, sqrt, log, pi
from scipy.special import erf, gamma
from scipy.stats import norm, weibull_min, uniform
from scipy.optimize import brentq, newton
from util.traits.editors.mpl_figure_editor import MPLFigureEditor
from matplotlib.figure import Figure
import string

from etsproxy.traits.ui.table_column import \
    ObjectColumn
    

from etsproxy.traits.ui.tabular_adapter \
    import TabularAdapter, AnITabularAdapter
from functools import reduce

class RunTableAdapter ( TabularAdapter ):

    columns = Property

    #---------------------------------------------------------------
    # EXTRACT FACTOR NAMES AS COLUMNS FOR TABLE EDITOR
    #-------------------------------------------------------------------
    def _get_columns(self):
        cols = [ ( 'run', 'index' ) ]
        ps = self.object
        for factor_name, factor in zip( ps.factor_names, ps.factor_list ):
            if isinstance( factor, SimEnumFactor ):
                cols.append( ( factor_name, 'enum' ) )
            elif isinstance( factor, SimIntFactor ):
                cols.append( ( factor_name, 'int' ) )                
            elif isinstance( factor, SimFloatFactor ):
                cols.append( ( factor_name, 'float' ) )                
        for output_name in ps.output_names:
            cols.append( (output_name, 'output') )
        
        return cols

    font        = 'Courier 10'
    alignment   = 'right'
    odd_bg_color     = 'lightblue'
    
    index_width = Float( 50 )
    index_image = Property
    def _get_index_image( self ):
        if self.object.output_table[ self.row,0 ]:
            return '@icons:red_ball'
        else:
            return None
            
    index_text  = Property
    def _get_index_text ( self ):
        return str( self.row )
    
    float_width = Float( 100 )
    float_text = Property
    def _get_float_text( self ):
        factor_idx = self.column - 1
        value = self.object.input_table[ self.row, factor_idx ]
        return '%6.2f' % value
        
    int_width   = Float( 60 )
    int_text    = Property
    def _get_int_text( self ):
        factor_idx = self.column - 1
        value = self.object.input_table[ self.row, factor_idx ]
        return '%d' % value
    
    enum_width  = Float( 120 )
    enum_text   = Property
    def _get_enum_text( self ):
        factor_idx = self.column - 1
        value = self.object.input_table[ self.row, factor_idx ]        
        return str( value )

    output_width  = Float( 120 )
    output_text   = Property
    def _get_output_text( self ):
        col_idx = self.column - 1
        value = self.object.run_table[ self.row, col_idx ]
        return str( value )

input_table_editor = TabularEditor( adapter = RunTableAdapter() )

class SimArray( HasTraits ):
    '''
    Parametric study in a model design space.
    
    This is a view to a parametric study on a supplied model.
    
    The model can be an arbitrary class that specifies a 
    * set of input factors and default ranges to be explored
    * set of output specifiers
    * peval function that returns an array of specified output values.
    
    The factors identified in the model are used to construct
    the factor ranges. By default, each factor is set to inactive 
    so that the parametric study outputs in a single-value level
    for each factor.
    
    The user can view any of the factors and specify if it should 
    be varied and also provide the levels.    
    '''
    implements( ISimArray )
    
    sim_model = Instance( ISimModel )
    def _sim_model_default(self):
        from .sim_model import SimModel
        return SimModel()    
    
    #---------------------------------------------------------------
    # FACTOR LIST SPECIFICATION
    #-------------------------------------------------------------------
    factor_dict = Dict
    def _factor_dict_default(self):
        return self._get_factor_dict()
    def _sim_model_changed(self):
        self.factor_dict = self._get_factor_dict()
                                                 
    def _get_factor_dict(self):
        '''
        Get the dictionary of factors provided by the simulation model.
        
        The factors are identified by the factor_levels metadata in the trait
        definition. For example
        
        my_param = Float( 20, factor_levels = (0, 10, 6) )
        
        specifies a float factor with  the levels [0,2,4,6,8,10]
        '''
        traits = self.sim_model.class_traits( ps_levels = lambda x: x != None )
        factor_dict = {}
        for tname, tval in list(traits.items()):
            if tval.is_trait_type( Int ):
                min_l, max_l, n_l = tval.ps_levels
                pt = SimIntFactor( min_level = min_l, max_level = max_l, n_levels = n_l )
            elif tval.is_trait_type( Float ):
                min_l, max_l, n_l = tval.ps_levels
                pt = SimFloatFactor( min_level = min_l, max_level = max_l, n_levels = n_l )
            else:
                pt = SimEnumFactor( model = self.sim_model, 
                                        levels = tval.ps_levels )
                
            factor_dict[tname] = pt
        return factor_dict
    
    n_factors = Property
    def _get_n_factors(self):
        return len( self.factor_dict )
    
    # alphabetically ordered names of factor ranges
    # 
    factor_names = Property( depends_on = 'factor_dict' )
    @cached_property
    def _get_factor_names(self):
        names = list(self.factor_dict.keys())
        names.sort()
        return names

    # alphabetically ordered list of factor ranges
    # 
    factor_list = Property( depends_on = 'factor_dict' )
    @cached_property
    def _get_factor_list(self):
        return [ self.factor_dict[ name ] for name in self.factor_names ]

    #---------------------------------------------------------------
    # SELECTED FACTOR FOR EDITING
    #-------------------------------------------------------------------
    # Edit factor levels to be associated inn the study
    #
    selected_factor_name   = Str( factor_modified = True, transient = True )
    def _selected_factor_name_default(self):
        return self.factor_names[0]
    def _selected_factor_name_changed(self):
        self.selected_factor = self.factor_dict[ self.selected_factor_name ]
    selected_factor  = Instance( SimFactor, transient = True )
    def _selected_factor_default(self):
        return self.factor_dict[ self.factor_names[0] ]

    #---------------------------------------------------------------
    # FULL FACTORIAL SPECIFICATION
    #-------------------------------------------------------------------
    #
    levels2run  = Property( Array( 'int_' ) )
    def _get_levels2run(self):
        '''Get the mapping between index of the factor level
           and the index of the run.

        levels2run[ factor1_level_idx, factor2_level_idx, ... ] = run_idx 

        '''
        # get number of levels for each factor
        #
        n_levels_list = [ factor.get_n_levels() for factor in self.factor_list ]
        size = reduce( lambda x,y: x*y, n_levels_list )
        levels_sizes = tuple( [ n_levels for n_levels in n_levels_list ] )
        levels2run = arange( size , dtype = 'int_' )
        levels2run = levels2run.reshape( levels_sizes ) 
        return levels2run
        
    run2levels = Property( Array( 'int_' ) )
    def _get_run2levels(self):
        '''Get the table of runs with rows containing the level indexes 
        within the factor levels list.
        
        run2levels[ run_idx ] = [ factor1_level_idx, factor2_level_idx, ... ] 
        '''
        n_levels_list = [ factor.get_n_levels() for factor in self.factor_list ]
        levels_slices = tuple( [ slice(0, n_levels, 1 ) for n_levels in n_levels_list ] )
        levels_grid = mgrid[ levels_slices ]
        
        run_idx_arr = c_[ tuple([ x.flatten() for x in levels_grid ]) ]        
        return run_idx_arr

    changed = Event
    @on_trait_change( 'factor_dict.+levels_modified,_output_cache' )
    def _set_changed(self):
        print('new values calculated')
        self.changed = True

    # Get the permutation of all factor levels as an array
    #
    input_table = Property( Array, depends_on = 'factor_dict.+levels_modified' )
    @cached_property
    def _get_input_table(self):
        '''Get the array containing the level values for each run.
        '''
        run_idx_arr = self._get_run2levels()
        run_levels_arr = zeros( run_idx_arr.shape, dtype = object )

        # construct the mapping between the indices of a factor 
        # and level and the level value
        level_map = [[ level 
                       for level  in factor.level_list ] 
                       for factor in self.factor_list ]
        
        for run_idx, run in enumerate( run_idx_arr ):
            for factor_idx, level_idx in enumerate( run ):
                run_levels_arr[ run_idx, factor_idx ] = level_map[ factor_idx ][ level_idx ] 

        return run_levels_arr

    #---------------------------------------------------------------
    # RESULT ARRAY SPECIFICATION
    #-------------------------------------------------------------------
    outputs = Property( List( SimOut ), depends_on = 'sim_model' )
    @cached_property
    def _get_outputs(self):
        if self.sim_model == None:
            return []
        else:
            return self.sim_model.get_sim_outputs()

    # extract the available names
    output_names = Property( List( Str ) )
    def _get_output_names(self):
        return [ r.name for r in self.outputs ]

    # number of outputs
    n_outputs = Property( Int )
    def _get_n_outputs(self):
        return len( self.output_names )

    #-------------------------------------------------------------------------
    # SELECTED RESULT FOR EDITTING
    #-------------------------------------------------------------------------
    # active selection
    #
    selected_output_name   = Str( transient = True )
    def _selected_output_name_default(self):
        return self.output_names[0]
    selected_output_idx    = Property( Int, depends_on = 'selected_output_name' )
    def _get_selected_output_idx(self):
        return self.output_names.index( self.selected_output_name )
    def _selected_output_name_changed(self):
        self.selected_output = self.selected_outputs[ self.selected_output_idx ]    
    selected_output  = Instance( SimOut, transient = True )
    def _selected_output_default(self):
        return self.outputs[ 0 ]
                
    #-------------------------------------------------------------------------
    # OUTPUT ARRAY
    #-------------------------------------------------------------------------
    output_array = Property( Array, depends_on = 'factor_dict.+levels_modified,_output_cache' )
    @cached_property
    def _get_output_array(self):
        '''Setup an array to accommodate the calculated values.
        
        The shape of the array respects the parametric space.
        Each dimension corresponds to a factor and has 
        the n_levels number of indices.
        
        The type of the array is object in order to allow the
        output values in terms complex objects
         - functions, tensors and fields - not only scalar 
        values.
        '''
        # get number of levels for each factor
        n_levels_list = [ factor.get_n_levels() for factor in self.factor_list ]
        rarray_shape = tuple( [ n_levels for n_levels in n_levels_list ] +
                              [ self.n_outputs ] )
        output_array = zeros( rarray_shape , dtype = object )

        # reuse the values from the cache
        #
        for run_idx, levels in enumerate( self.input_table ):
            outputs = self._output_cache.get( tuple(levels), None )
            if outputs != None:
                #print 'cached value for run', run_idx
                levels_idx = self.run2levels[ run_idx ]
                output_array[ tuple( levels_idx ) ] = outputs
        return output_array

    def _get_slice_n_sims( self, factor_slices ):
        '''Determine the number of sims involved in the given slice.
        '''
        run_idx_list = self.levels2run[ factor_slices ].flatten()
        return len( run_idx_list )
    
    #def _get_slice_runs( self, factor_slices ):
        
    #---------------------------------------------------------------
    # RESULT DATA ARRAY / DICT
    #-------------------------------------------------------------------
    # cached management of the calculated data
    # parallel scheduling of computational runs
    #
    _output_cache = Dict

    def __getitem__( self, factor_slices ):
        '''
        Access to the output_array using factor level indices.

        This method enables access to the values using the syntax
        
        output_sub_array = pstudy[ f1_level_idx, f2_level_idx, ... ]
        
        Here the standard numpy indices including slice and elipses can be used. 
        '''

        # map the slices within the levels2run array
        # to the indices of the expanded input_table
        #
        n_sims = self._get_slice_n_sims( factor_slices )
        progress = ProgressDialog( title = 'simulation progress',
                                   message = 'running %d simulations' % n_sims,
                                   max = n_sims,
                                   show_time = True,
                                   can_cancel = True )
        progress.open() 
        
        run_idx_list = self.levels2run[ factor_slices ].flatten()
        runs_levels     = self.input_table[ run_idx_list ]
        runs_levels_idx = self.run2levels[ run_idx_list ]

        # start the computation for each of the runs
        #
        sim_idx = 0
        for run_levels, run_levels_idx in zip( runs_levels, runs_levels_idx ):

            # check to see if this run is already in the cache 
            #
            outputs = self._output_cache.get( tuple( run_levels ), None )
            if outputs == None:

                print('new simulation', sim_idx)

                # Set the factor values of the run in 
                # the simulation model
                #
                for factor_name, factor, level in zip( self.factor_names,
                                                       self.factor_list, 
                                                       run_levels ):
                    level = factor.get_level_value( level )
                    setattr( self.sim_model, factor_name, level )
                    
                # Perform the simulation
                #
                outputs = self.sim_model.peval()
                
                self.output_array[ tuple( run_levels_idx ) ] = outputs
                self._output_cache[ tuple(run_levels) ] = outputs

            else:
                print('cached simulation', sim_idx)

            # let the progress bar interfere
            #
            (cont, skip) = progress.update( sim_idx )
            if not cont or skip:
                break
            sim_idx += 1

        progress.update( n_sims )
        return self.output_array[ factor_slices ]        

    n_runs = Property( Int, depends_on = 'factor_dict.+levels_modified' )
    @cached_property
    def _get_n_runs(self):
        return self.input_table.shape[0]

    fraction_cached = Property( Float, depends_on = 'factor_dict,_output_cache' )
    @cached_property
    def _get_fraction_cached(self):
        return float( len( self._output_cache) ) / float( len( self.run_table ) )
    
    output_table = Property( Array(object), 
                             depends_on = 'factor_dict.+levels_modified,_output_cache' )
    @cached_property
    def _get_output_table(self):
        '''
        Expand the ouput_array into two-dimensional table with
        first index representing the run and second index the output

        @todo: this can be probably simplifed by an index mapping 
             - check examples in numpy
        '''
        output_table = zeros( (self.n_runs, self.n_outputs), dtype = object )
        for run_idx, levels in enumerate(self.run2levels ):
            output_table[ run_idx, : ] = self.output_array[ tuple(levels ) ] 
        return output_table

    run_table = Property( Array(object),
                          depends_on = 'factor_dict.+levels_modified,_output_cache' )
    @cached_property
    def _get_run_table(self):
        '''
        Glue together the input and output tables.
        '''
        return hstack( [ self.input_table, self.output_table ] )

    def clear_cache(self):
        ''' Clear the output cache '''
        self._output_cache = {}

    def save(self, file ):
        pickle.dump( self, file )

    def load(self, file ):
        self = pickle.load( file )

    #---------------------------------------------------------------------------------
    # VIEW
    #---------------------------------------------------------------------------------

    traits_view = View( 
                    HSplit( 
                           VGroup(
                               VGroup(
                                     Item( 'selected_factor_name', 
                                           editor = EnumEditor( name = 'factor_names' ),
                                           show_label = False ),
                                     Item( 'selected_factor@',
                                           resizable = True,
                                           show_label = False ),
                                     label = 'factor levels',
                                     id = 'sim_pstudy.view_model.factor',
                                     dock = 'tab',
                                     scrollable = True,
                                     ),
                                VGroup(
                                     Item( 'output_name',
                                           editor = EnumEditor( name = 'output_names' ), 
                                           show_label = False ),
                                     Item( 'output@',
                                           show_label = False,
                                           springy = True ),
                                     label = 'vertical axis',
                                     id = 'sim_psrudy.viewmodel.control',
                                     dock = 'tab',
                                   ),
                                id = 'sim_pstudy.viewmodel.factor',
                                layout = 'split',
                                label = 'plot range specification',
                                dock = 'tab',
                                ),
                           VSplit( 
                                   VGroup(
                                         HGroup(Item('fraction_cached',
                                                     style = 'readonly',
                                                     label = 'cached [%]'),
                                                Item('n_runs',
                                                     style = 'readonly',
                                                     label = 'number of runs' )
                                                ),
                                         Item( 'run_table',
                                               editor = input_table_editor, 
                                               show_label = False,
                                               style = 'custom' ),
                                         label = 'run table',
                                         id = 'sim_psrudy.viewmodel.input_table',
                                         dock = 'tab',
                                         ),                                         
                                   id = 'sim_pstudy.viewmodel.right',                                       
                                 ),
                            id = 'sim_pstudy.viewmodel.splitter',        
                            #group_theme = '@G',
                            #item_theme  = '@B0B',
                            #label_theme = '@BEA',                                      
                        ),
                        id = 'sim_pstudy.viewmodel',
                        dock = 'tab',                        
                        resizable = True,
                        height = 0.8, width = 0.8,
                        ) 
            
def run():
    from .sim_model import SimModel
    sim_array = SimArray( sim_model = SimModel() )
    print(sim_array.levels2run[ :,0, 0, 0 ])
    print(sim_array[ :,0, 0, 0 ])
    print(sim_array.run_table[0])
    print('array_content', sim_array.output_array[0,0,0,0,:])
    sim_array.clear_cache()
    print('array_content', sim_array.output_array[0,0,0,0,:])
    sim_array.configure_traits()
    
if __name__ == '__main__':
    run()                 


# Import Standard Libraries
import logging
import scipy as np
import matplotlib.pyplot as plt

# Import Local Libraries
import Util_ACI as ACI
import Util_EC2 as EC2
from PSC import PrestressedBeam

#===========================================================================
#   T Beam
#===========================================================================


class TeeBeam(PrestressedBeam):
	bf = 1200
	hf = 150

	bw = 300
	H = 1200

	d_S = 1000
	d_P = 1000
	d_P0 = 500

	As = 0
	Ap = 2400

	fck = 30
	fyk = 500
	fpk = 1860

	Es = 205000
	k = 0.01
	Ep = 195000
	Pm = 2772000
	units = "MPa"

    #   |<------- bf ------->|
    #   ______________________  ___
    #   |                |   |   |
    #   |               hf   |   |   ___
    #   |_______      ___|___|   |    |
    #          |      |  |       |    |
    #          |<-bw->|  |       |    |
    #          |      |  |       H    |   ___
    #          |      |  hw      |    yf   |
    #          |      |  |       |    |    yw
    #          |      |  |       |    |    |
    #          |______| _|_     _|_  _|_  _|_

	def __init__(self):

		PrestressedBeam.__init__(self)

        # Properties of flange
		Af = self.hf*self.bf
		If = self.bf*self.hf**3/12
		yf = self.H - self.hf/2
		logging.debug("    Af = {:6.2f}, If = {:10.2f}".format(Af,If))

		# Properties of web
		hw = self.H - self.hf
		Aw = self.bw*hw
		Iw = self.bw*hw**3/12
		yw = hw/2
		logging.debug("    Aw = {:6.2f}, Iw = {:10.2f}".format(Aw, Iw))

		# Properties of TeeBeam
		self.Ac = Af + Aw
		self.yb = (Af*yf+Aw*yw)/self.Ac
		self.yt = self.H - self.yb
		logging.debug("    yt = {:6.2f}, yb = {:}".format(self.yb, self.yt))

		self.e_P = self.d_P - self.yt
		logging.debug("    e_P = {:6.2f}".format(self.e_P))

		self.I = Iw + If + Af*(self.yb - yf)**2 + Aw*(self.yb - yw)**2
		logging.debug("    Ac = {:6.2f}, I = {:10.2f}".format(self.Ac, self.I))

		self.Wt = self.I/self.yt
		self.Wb = self.I/self.yb

		logging.debug("    Wt = {:6.2f}, Wb = {:6.2f}".format(self.Wt, self.Wb))

	def plotTeeBeam(self):
		x1 = self.bf
		x2 = (self.bf + self.bw)/2
		x3 = (self.bf - self.bw)/2

		y0 = self.H
		y1 = self.H - self.hf
		X = [0, x1, x1, x2, x2, x3, x3, 0, 0]
		Y = [y0, y0, y1, y1, 0, 0, y1, y1, y0]

		ax = self.fig.add_subplot(131)
		ax.plot(X, Y, linewidth=1, color='k')
		ax.fill(X, Y, color = 'lightblue')

		ax.plot([0, self.bf], [self.yb, self.yb], linestyle='--', color='k')

		ax.plot(self.bf/2, self.yb, marker='+', color='k')

		if self.Ap != 0:
			ax.plot(self.bf/2, self.H - self.d_P, marker='o', color='r')
			ax.plot(self.bf/2, self.H - self.d_P0, marker='o', color='r')

		if self.As != 0:
			ax.plot(self.bf/2, self.H - self.d_S, marker='o', color='k')

		plt.show()

	def plotStressDist(self, Xu, *args):

		ax = self.fig.add_subplot(132)
		ax.plot([0, 0], [0, self.H], linewidth=2, color='k')

		# Moment Capacity
		if Xu < self.hf:
			Nc1 = args[0]
			print(Nc1)
		elif Xu > self.hf:

			[fcd, Nc1, y1, Nc2, y2, Ns, y3] = args

			h0 = self.H - self.hf
			self.__plotConcreteStress(ax, fcd, self.H, h0)

			h1 = self.H - Xu
			self.__plotConcreteStress(ax, fcd, self.H, h1)

			print(Ns)

			h1 = h1 + Xu/2

			plt.xlim((-1.2*fcd, 1.2*fcd))
			plt.show()


if __name__ == "__main__":
	logging.basicConfig(level=logging.DEBUG, format='%(message)s')
	beam = TeeBeam()
	beam.EC2_design_moment()
	beam.plotTeeBeam()
	# beam.plotStressDist()



'''
Created on 18. 3. 2014

@author: Vancikv
'''

import os

def get_outfile(folder_name, file_name):
    '''Returns a file in the specified folder using the home
    directory as root.
    '''
    HOME_DIR = os.path.expanduser("~")
    out_dir = os.path.join(HOME_DIR, folder_name)
    if not os.path.exists(out_dir):
        os.makedirs(out_dir)
    outfile = os.path.join(out_dir, file_name)
    return outfile



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Feb 3, 2010 by: rch

if __name__ == '__main__':
    
    # DEFINE A SIMULATION MODEL
    # -------------------------
    # import a model - a class that provides 
    # - factors         as traits with a prescribed set of levels
    #                   that should be included in the parametric study.
    #                   A trait is regarded as Factor if it specifies
    #                   a set of levels in form ps_levels = True.
    # - get_sim_outputs()   as instances of SimOut class
    #                   defining the name and order of outputs
    # - peval()          method returning a vector of results
    #                   the order of outputs is prescribed by the
    #                   specification given in the get_outputs() method 
    #
    # Here we just import predefined Foo model with three four inputs
    #
    # [ index_1, material_model, param_1, param_2 ]
    # 
    # The inputs have the types [ Int, Callable, Float, Float
    # 
    from .sim_model import SimModel
    sim_model = SimModel()
    
    # The model response is obtained by issueing
    #    
    print('default evaluation', sim_model.peval())
       
    # returning an array with two values. 
    # In this call, the default values of the factors 
    # [input_1, material_model, param_1, param_2 ]
    # were taken. The factor levels were ignored.
    
    # DEFINING A STUDY
    # --------------
    # In order to study the response of the model in a broader range 
    # of specified levels we now construct the parametric study
    #
    from .sim_pstudy import SimArray
    pstudy = SimArray( sim_model = sim_model )
    
    # ACCESSING OUTPUTS
    # --------------
    # The pstudy can be regarded as an n-dimensional array
    # providing the results for each combination of factor levels
    # In order to get the model response for the first level
    # of all parameters the index operator can be used as follows:
    #
    print('model output for ground levels', pstudy[0,0,0,0])
    
    # The combination of last levels is obtained as
    #
    print('model output for floor levels', pstudy[-1,-1,-1,-1])
    
    # The computation of outputs is performed on demand and cached. 
    # Thus, if an index appears the second time only the cached value
    # is returned.
    #
    print('lookup the output in the cache', pstudy[-1,-1,-1,-1])
    
    # Just like for any array, indexes may be sliced
    #
    print('get the outputs for all levels of index_1', pstudy[:,0,0,0])
    
    # the result of this call is 2-dimensional array with the first index
    # specifying the level of index_1 and second index giving the output
    
    # In analogy, for slice over the last two indexes
    #
    print('get the outputs for all combinations of param_1 x param_2', pstudy[0,0,:,:])
    
    # a 3-dimensional array is returned with first two indexes correspond
    # to the levels of param_1 and param_2 and third index identifying the output
    
    # Finally, the whole study can be performed using ellipsis
    #
    print('get all the values in the n-dimensional space', pstudy[...])
    
    # The result is a 5-dimensional array with first four indexes 
    # denoting the factor levels and last index the output.
    # Note that the previously accessed slices are reused in this call. 
    
    # MODIFYING LEVELS
    # ----------------
    # The initially specified set of default levels for each factor
    # can be changed within an existing study.
    #
    pstudy.factor_dict['param_1'].max_level = 10
    pstudy.factor_dict['param_1'].n_levels = 5
    print('get output for first two levels of param_1', pstudy[-1,-1,1,-1])
    
    # Note that values for pstudy[:,:,0,:] were included in the old
    # grid of levels and are reused in the new study as well.
    
    # COMMENTS
    # --------
    # The study is limited to an a regular grid of levels. This can be regarded 
    # as a full factorial in the nomenclature of the Design of Experiments (DOE).
    # The table showing the full factorial in the user interface can be seen by issuing
    #
    pstudy.configure_traits()
     
    # The evaluation of the model is not performed for all possible 
    # combinations of factor levels. The evaluation is done first when
    # the particular value is accessed using the level indexes.
    #
    # The numpy indexing functions are used to access values in the
    # factor space. The data structure can be used for 
    # - on demand construction of 2D and 3D views into the output 
    #   space of the study
    # - construction of a regression model within the factor space in analogy 
    #   to the DOE
    # - supporting adaptive integration procedures within the factor space
    #   as it is the case in statistical analysis  of multi-variate response. 
    #
    # An example of the first application can be demonstrated by using the
    # SimArrayView class
    #
    from .sim_array_view import SimArrayView
    SimArrayView( model = pstudy ).configure_traits()
    
    # SAVING THE STUDY
    # ----------------
    # In order to save the study, it the SimPStudy class manages
    # the object persistence. It associates the study with a file name
    # and monitors whether the study has been changed or not.
    #
    from .sim_pstudy import SimPStudy
    sim_pstudy = SimPStudy( sim_model = sim_model )
    sim_pstudy.configure_traits()
    
    # RESETTING THE CACHE
    # -------------------
    
    # LOADING AN EXISTING STUDY
    # -------------------------
    
    


'''
Created on 15. 2. 2014

@author: Vancikv
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCTexLayer

import numpy as np

def test_cross_section_mn():
    '''Test the moment and normal force calculated for a cross section.
    Rectangular cross section with spread textile reinforcement.
    '''

    tl1 = RLCTexLayer(z_coord=0.1, material='default_fabric', material_law='fbm')
    tl2 = RLCTexLayer(z_coord=0.05, material='default_fabric', material_law='fbm')
    ge = MCSGeoRect(height=0.5, width=0.3)
    cs = CrossSection(reinf=[tl1, tl2],
                         matrix_cs=MatrixCrossSection(geo=ge,
                                        n_cj=20, material='default_mixture',
                                        material_law='constant'),
                         eps_lo=0.008,
                         eps_up=-0.0033,
                         )

    tl1.material_law_.set(sig_tex_u=1216., eps_u=0.014, m=0.5)
    tl1.material_.set(s_0=0.018, A_roving=0.461)

    assert np.allclose([cs.M, cs.N], [433.45620169134492, -2247.2883277004325])
    tl1.z_coord = 0.4
    assert np.allclose([cs.M, cs.N], [432.39449649331749, -2254.3663623539496])
    ge.width = 0.15
    assert np.allclose([cs.M, cs.N], [216.14717747037844, -1127.4335350583763])
    cs.eps_lo = 0.014
    assert np.allclose([cs.M, cs.N], [156.7097970367976, -734.53214309844532])

if __name__ == '__main__':
    test_cross_section_mn()



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Jan 19, 2010 by: rch


from traits.api import HasTraits, Float, Property, cached_property, \
                                Instance, List, on_trait_change, Int, Tuple, Bool, \
                                DelegatesTo, Event, Str, Button, Dict, Array, Any
from traitsui.api import \
    View, Item, Tabbed, VGroup, HGroup, ModelView, HSplit, VSplit, \
    CheckListEditor, EnumEditor, TableEditor, TabularEditor,  Handler

from etsproxy.traits.ui.menu import Action, CloseAction, HelpAction, Menu, \
                                     MenuBar, NoButtons, Separator, ToolBar                    

from etsproxy.traits.ui.tabular_adapter \
    import TabularAdapter
    
from etsproxy.pyface.api import ImageResource
    
from etsproxy.traits.ui.menu import \
    OKButton
from .i_sim_model import \
    ISimModel
from .sim_factor import \
    SimFactor
from .sim_output import \
    SimOut
from .i_sim_array import \
    ISimArray
from .sim_todo import \
    ToDo

from numpy import array, linspace, frompyfunc, zeros, column_stack, \
                    log as ln, append, logspace, hstack, sign, trapz, mgrid, c_, \
                    zeros
                    
from math import exp, e, sqrt, log, pi
from scipy.special import erf, gamma
from scipy.stats import norm, weibull_min, uniform
from scipy.optimize import brentq, newton
from util.traits.editors.mpl_figure_editor import MPLFigureEditor
from matplotlib.figure import Figure
import string

class SimArrayView( ModelView ):
    '''
    View into the parametric space constructed over the model.

    The is associated with the PStudySpace instance covering the
    factor ranges using an n-dimensional array.
    
    The view is responsible for transferring the response values
    into 2D and 3D plots. Depending on the current view specification
    it also initiates the calculation of response values in the 
    currently viewed subspace of the study. 
    '''

    model = Instance( ISimArray )
    
    #---------------------------------------------------------------
    # PARAMETER RANGE SPECIFICATION
    #-------------------------------------------------------------------
    factor_dict = DelegatesTo('model')

    # alphabetically ordered names of factors
    # 
    factor_names = DelegatesTo('model')

    # alphabetically ordered list of factors
    # 
    factor_list = DelegatesTo('model')

    #---------------------------------------------------------------
    # X-PARAMETER RANGE SPECIFICATION
    #-------------------------------------------------------------------
    # Selected factor name for evalution along the X-axis
    #
    x_factor_name   = Str( factors_modified = True )
    def _x_factor_name_default(self):
        return self.factor_names[0]
    def _x_factor_name_changed(self):
        if self.x_factor_name == self.y_factor_name:
            self.y_factor_name = '-'
        if self.x_factor_name == self.other_factor_name:
            self.other_factor_name = '-'
        self.x_factor = self.factor_dict[ self.x_factor_name ]
        
    x_factor  = Instance( SimFactor )
    def _x_factor_default(self):
        return self.factor_dict[ self.factor_names[0] ]
    
    # index of the currently selected variable 
    x_factor_idx = Property()
    def _get_x_factor_idx(self):
        return self.factor_names.index( self.x_factor_name )
    #---------------------------------------------------------------
    # Y-PARAMETER RANGE SPECIFICATION
    #-------------------------------------------------------------------
    y_factor_names = Property( depends_on = 'x_factor_name' )
    @cached_property
    def _get_y_factor_names(self):
        current_x_factor = self.x_factor_name
        current_x_factor_idx = self.factor_names.index( current_x_factor )
        y_factor_names = self.factor_names[:current_x_factor_idx] + \
                        self.factor_names[current_x_factor_idx+1:]
        return [ '-' ] + y_factor_names
    #
    # Selected factor name for evalution of multiple lines
    #
    y_factor_name   = Str( '-' , factors_modified = True )
    def _y_factor_name_changed(self):
        if self.y_factor_name == self.other_factor_name:
            self.other_factor_name = '-'
        if self.y_factor_name == '-':
            self.y_factor = None
        else:
            self.y_factor = self.factor_dict[ self.y_factor_name ]
    y_factor  = Instance( SimFactor )

    y_factor_idx = Property()
    def _get_y_factor_idx(self):
        return self.factor_names.index( self.y_factor_name )

    #------------------------------------------------------------------
    # OTHER PARAM LEVELS
    #------------------------------------------------------------------
    other_factor_names = Property( depends_on = 'x_factor_name, y_factor_name' )
    @cached_property
    def _get_other_factor_names(self):
        x_factor_idx = self.factor_names.index( self.x_factor_name )

        y_factor_idx = x_factor_idx
        if self.y_factor_name != '-':
            y_factor_idx = self.factor_names.index( self.y_factor_name )
        
        ignore_idx = [ x_factor_idx, y_factor_idx ]
        ignore_idx.sort()
        
        other_factor_names = self.factor_names[:ignore_idx[0] ] + \
                            self.factor_names[ignore_idx[0]+1: ignore_idx[1]] + \
                            self.factor_names[ignore_idx[1]+1:]

        return ['-'] + other_factor_names
    
    #
    # Selected factor name for evalution of multiple lines
    #
    other_factor_name   = Str( '-' , factors_modified = True )
    def _other_factor_name_changed(self):
        if self.other_factor_name == '-':
            self.other_factor_levels = []
        else:
            levels = self.factor_dict[ self.other_factor_name ].level_list
            self.other_factor_levels = list( levels )
            other_factor_idx = self.factor_names.index( self.other_factor_name )
            other_factor_level_idx = self.frozen_factor_levels[ other_factor_idx ]
            self.other_factor_level = self.other_factor_levels[ other_factor_level_idx ]
            
    other_factor_levels = List
    other_factor_level = Any
    def _other_factor_level_changed(self):
        level_idx = self.other_factor_levels.index( self.other_factor_level )
        other_factor_idx = self.factor_names.index( self.other_factor_name )
        self.frozen_factor_levels[ other_factor_idx ] = level_idx
    
    frozen_factor_levels = Array
    def _frozen_factor_levels_default(self):
        return zeros( self.model.n_factors, dtype = 'int_' )
    #---------------------------------------------------------------
    # OUTPUT ARRAY SPECIFICATION
    #-------------------------------------------------------------------
    outputs = DelegatesTo('model')

    # extract the available names
    output_names = DelegatesTo('model')

    # active selection to be plotted
    output_name   = Str
    def _output_name_default(self):
        return self.output_names[0]
    output_idx    = Property( Int, depends_on = 'output_name' )
    def _get_output_idx(self):
        return self.output_names.index( self.output_name )
    def _output_name_changed(self):
        self.output = self.outputs[ self.output_idx ]    
    output  = Instance( SimOut )
    def _output_default(self):
        return self.outputs[ 0 ]

    #---------------------------------------------------------------
    # PLOT OBJECT
    #-------------------------------------------------------------------
    figure = Instance(Figure)
    def _figure_default(self):
        figure = Figure( facecolor = 'white' )
        figure.add_axes([0.12, 0.13, 0.85, 0.74])
        return figure

    #---------------------------------------------------------------------------
    # Public Controller interface
    #---------------------------------------------------------------------------
    def start_study(self, ui_info):
        self._start_study()

    def stop_study(self, ui_info):
        todo = ToDo()
        todo.configure_traits( kind = 'modal' )

    def _start_study(self):
        
        # identify the runs to be performed
        # use slices along the varied factors 
        # to obtain the indices of the values.

        # get the sliced dimensions
        #
        factor_levels = [ level for level in self.frozen_factor_levels ]
        factor_levels[ self.x_factor_idx ] = slice( None )
        if self.y_factor_name != '-': 
            factor_levels[ self.y_factor_idx ] = slice( None )
        
        factor_slices = tuple( factor_levels )

        # get the response value for the given factor slices
        #
        output_array = self.model[ factor_slices ]
        
        # map the array dimensions to the plot axes
        #
        figure = self.figure
        
        axes = figure.axes[0]
        axes.clear()

        x_levels = self.x_factor.level_list

        if self.y_factor_name == '-':
            
            axes.plot( x_levels, output_array[ :, self.output_idx ]
                       # color = c, linewidth = w, linestyle = s 
                       )
        else:
            y_levels = self.y_factor.level_list
            for i_y, y_level in enumerate( y_levels ):
        
                index = x_levels

                # The dimensions of the returned array are given
                # by the index of the factors within the pstudy
                # In other words, the subspace to be plotted has
                # the same order of factors as the original space.
                # The remapping of the axes must therefore respect
                # this order and take y data from the first dimension
                # if y_factor_idx is lower than y_factor_idx
                # 
                if self.y_factor_idx > self.x_factor_idx:
                    values = output_array[ :, i_y, self.output_idx ]
                else:
                    values = output_array[ i_y, :, self.output_idx ]

                axes.plot( index, values
                           # color = c, linewidth = w, linestyle = s 
                           )
            legend = [ str(level) for level in y_levels ]
            axes.legend( legend, loc = 'best')
            
        axes.set_xlabel( '%s [%s]' % (self.x_factor_name, self.x_factor.unit), 
                         weight = 'semibold')
        axes.set_ylabel( '%s [%s]' % (self.output_name, self.output.unit),
                         weight = 'semibold')
        
#        axes.set_title( 'strength size effect',\
#                        size = 'large', color = 'black',\
#                        weight = 'bold', position = (.5,1.03))
        axes.set_axis_bgcolor(color = 'white')
#        axes.ticklabel_format(scilimits = (-3.,4.))
        axes.grid(color='gray', linestyle='--', linewidth=0.1, alpha = 0.4)
#        axes.legend(( legend ), loc = 'best')
#        axes.set_xscale('log' ) #, subsx = [0, 1, 2, 3 ] )
#        axes.set_yscale('log' ) # , subsy = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9] )
        #axes.set_xlim(10.e-4)

        self.data_changed = True

    data_changed = Event
         
    traits_view = View( 
                    HSplit( 
                            VGroup(
                                VGroup(
                                 Item( 'x_factor_name', 
                                       editor = EnumEditor( name = 'factor_names' ),
                                       label = 'horizontal axis' ),
                                 Item( 'y_factor_name', 
                                       editor = EnumEditor( name = 'y_factor_names' ),
                                       label = 'depth axis' ),
                                 Item( 'other_factor_name', 
                                       editor = EnumEditor( name = 'other_factor_names' ),
                                       label = 'other factors' ),
                                 Item( 'other_factor_level', 
                                       editor = EnumEditor( name = 'other_factor_levels' ),
                                       label = 'viewed level' ),
                                 label = 'viewed subspace',
                                 id = 'sim_pstudy.viewmodel.factor.subspace',
                                 dock = 'tab',
                                 scrollable = True,
                                 ),
                                VGroup(
                                 Item( 'output_name',
                                       editor = EnumEditor( name = 'output_names' ), 
                                       show_label = False ),
                                 Item( 'output@',
                                       show_label = False,
                                       springy = True ),
                                 label = 'vertical axis',
                                 id = 'sim_psrudy.viewmodel.control',
                                 dock = 'tab',                                     
                                 ),
                            id = 'sim_pstudy.viewmodel.left',
                            label = 'studied factors',
                            layout = 'normal',
                            dock = 'tab',
                            ),                                                                               
                               VSplit( 
                                    VGroup(
                                        Item('figure',  editor=MPLFigureEditor(),
                                             resizable=True, show_label = False ),
                                             label = 'plot sheet',
                                            id = 'sim_pstudy.viewmode.figure_window',
                                            dock = 'tab',
                                        ),
                                   id = 'sim_pstudy.viewmodel.right',                                       
                                 ),
                            id = 'sim_pstudy.viewmodel.splitter',        
                            #group_theme = '@G',
                            #item_theme  = '@B0B',
                            #label_theme = '@BEA',                                      
                        ),
                        toolbar = ToolBar( 
                                          Action(name="Run",
                                                 tooltip = 'Start computation',
                                                 image = ImageResource('kt-start'),
                                                 action="start_study"),
                                          Action(name="Pause",
                                                 tooltip = 'Pause computation',
                                                 image = ImageResource('kt-pause'),
                                                 action="pause_study"),
                                          Action(name="Stop",
                                                 tooltip = 'Stop computation',
                                                 image = ImageResource('kt-stop'),
                                                 action="stop_study"),
                                          image_size = (32,32),
                                          show_tool_names = False,
                                          show_divider = True,
                                          name = 'view_toolbar' ),                                         
                        title = 'SimVisage Component: Parametric Studies',
                        id = 'sim_pstudy.viewmodel',
                        dock = 'tab',                        
                        resizable = True,
                        height = 0.8, width = 0.8,
                        buttons = [OKButton]) 
            
def run():
    from .sim_model import SimModel
    from .sim_array import SimArray

    pstudy = SimArray( sim_model = SimModel() )
    pstudy_view = SimArrayView( model = pstudy )
    pstudy_view.configure_traits()
    
if __name__ == '__main__':
    run()                 


"""
Traits UI editor for WX, based on the Chaco PlotEditor in
traits.ui.wx.plot_editor.
"""

# Enthought library imports
from enable.api import black_color_trait, LineStyle, ColorTrait, \
    white_color_trait

# from etsproxy.enable.wx_backend.api import Window
# <=== Unchange

from traits.api import false, Str, Range, Float, Bool, Int, Any, \
    List, HasPrivateTraits, Instance
from traitsui.api import Item, UI
from traitsui.wx.editor import Editor
from traitsui.wx.editor_factory import EditorFactory
from traitsui.menu import Action, ToolBar, Menu
#from etsproxy.traits.ui.wx.helper import traits_ui_panel

# Local relative imports
from chaco.plot_containers import OverlayPlotContainer
from chaco.plot_label import PlotLabel

# Somewhat unorthodox...

# from etsproxy.chaco.tools.api import SimpleZoom, DataLabelTool
# <=== Unchange

from chaco.api import OverlayPlotContainer, PlotLabel, Label, \
    DataLabel, TextBoxOverlay
from chaco.polar_line_renderer import PolarLineRenderer
from chaco.abstract_plot_renderer import AbstractPlotRenderer
from chaco.polar_mapper import PolarMapper
from chaco.array_data_source import ArrayDataSource
from chaco.data_range_1d import DataRange1D

from numpy import frompyfunc, add, ndarray, arange, array, compress, \
    concatenate, cos, pi, sin, transpose, zeros

import wx

from pyface.image_resource import ImageResource
from pyface.api import FileDialog, OK

# Constant variable
WindowColor = "lightgray"


WILDCARD = "Saved plots (*.eps)|*.eps|"\
           "All files (*.*)|*.*"

# Range for the height and width for the plot widget.
PlotSize = Range(50, 1000, 180)


def coord_trans_plt(r_value, frac_noplot, plotrange_min, plotrange_max):
    r_a = frac_noplot
    if r_value < plotrange_min:
        r_value_plt = r_a
    elif r_value > plotrange_max:
        r_value_plt = 1.0
    else:
        r_value_plt = r_a + \
            (1. - r_a) * \
            (r_value - plotrange_min) / \
            (plotrange_max - plotrange_min)
    return r_value_plt


def _get_radius_arr_plt(radius_arr, frac_noplot, plotrange_min, plotrange_max):
    vcoord_trans_plt = frompyfunc(coord_trans_plt, 4, 1)
    radius_arr_plt = array(vcoord_trans_plt(radius_arr, frac_noplot,
                                            plotrange_min, plotrange_max),
                           dtype='float_')
    return radius_arr_plt


def unitcircle_fn(theta_value, frac_noplot, plotrange_min, plotrange_max):
    r_a = coord_trans_plt(0.0, frac_noplot, plotrange_min, plotrange_max)
    unitcircle_plt = r_a
    return unitcircle_plt


def _get_unitcircle_arr(theta_arr, frac_noplot, plotrange_min, plotrange_max):
    vunitcircle_fn = frompyfunc(unitcircle_fn, 4, 1)
    return array(vunitcircle_fn(theta_arr, frac_noplot, plotrange_min,
                                plotrange_max), dtype='float_')


class MFnPolarPlotItem(Item):

    """ A Traits UI Item for a Chaco plot, for use in Traits UI Views.

    NOTE: ComponentEditor is preferred over this class, as it is more
    flexible.
    """
    # Name of the trait that references the index data source.
    index = Str
    # Name of the trait that references the value data source.
    value_list = List(Str)

    # Foreground olor of the plot.
    color = ColorTrait("green")
    # Background color of the plot.
    bgcolor = white_color_trait

    # Width of the plot border
    border_width = Int(1)
    # Is the border visible?
    border_visible = false
    # Color of the border.
    border_color = black_color_trait

    # The type of the plot as a string.
    type_trait = Str

    def __init__(self, index, value_list, type="line", **traits):
        self.index = index
        self.value_list = value_list
#        self.type = type
        self.name = "Plot"

        super(Item, self).__init__(**traits)
        self.editor = MFnPolarEditorFactory()
        self.editor.plotitem = self

        return


#    def _set_bg_color(self, val):
#        self.bgcolor = val
#
#    def _get_bg_color(self):
#        return self.bgcolor


class MFnPolarEditorFactory (EditorFactory):

    # Width of the plot editor.
    width = PlotSize
    # Height of the plot editor.
    height = PlotSize
    # The ChacoPlotItem associated with this factory.
    plotitem = Any

    def simple_editor(self, ui, object, name, description, parent):
        return MFnPolarPlotEditor(parent,
                                  factory=self,
                                  ui=ui,
                                  object=object,
                                  name=name,
                                  description=description)


class MFnPolarPlotEditor (Editor):

    """ Traits UI editor for displaying trait values in a Chaco plot.
    """

    # TableEditorToolbar associated with the editor:
    toolbar = Any

    # The Traits UI associated with the table editor toolbar:
    toolbar_ui = Instance(UI)

    # flag that indicates whether the unitcircle is to be plotted
    # (i.e. if zero is in the plotrange)
    plot_unitcircle_flag = Bool(False)

    def validate(self, plotrange_min, plotrange_max):
        if plotrange_min >= plotrange_max:
            print("### Error ###: invalid plot ranges. plotrange_max must be \
                   greater than plotrange_min.")
            return False
        else:
            return True

    def init(self, parent):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        factory = self.factory
        plotitem = factory.plotitem

        container = OverlayPlotContainer(use_backbuffer=True)
        self._container = container

        window = Window(parent, component=container)
        self.control = control = window.control


# new lines for Toolbar
#        panel = traits_ui_panel( parent, -1 )
##
# super(MFnPolarPlotEditor,self).init(panel)
##
#        plot_control = self.control
##        self.control = panel
##
# sizer        = wx.BoxSizer( wx.HORIZONTAL) #VERTICAL )
#        self._create_toolbar( panel, sizer )
##
#        sizer.Add( plot_control, 1, wx.ALIGN_RIGHT | wx.EXPAND )
##
#        panel.SetSizer( sizer )
####################################

        control.SetSize((factory.width, factory.height))
        object = self.object

        # Attach listeners to the object's traits appropriately so we can
        # update the plot when they change.  For the _update_axis_grids()
        # callback, we have to wrap it in a lambda to keep traits from
        # inferring the calling convention based on introspecting the argument
        # list.
        for name in [plotitem.index] + plotitem.value_list:
            print("setting on trait change for ", name)
            object.on_trait_change(self._update_data, name)
        object.on_trait_change(self.update_editor, plotitem.type_trait)

        return

    #-------------------------------------------------------------------------
    #  Creates the table editing tool bar:
    #-------------------------------------------------------------------------

    def _create_toolbar(self, parent, sizer):
        """ Creates the table editing toolbar.
        """
        factory = self.factory

        toolbar = MFnPolarPlotEditorToolbar(parent=parent, editor=self)
        tb_sizer = wx.BoxSizer(wx.HORIZONTAL)

        self.toolbar = toolbar
        tb_sizer.Add(toolbar.control, 0)
        tb_sizer.Add((1, 1), 1, wx.EXPAND)

        sizer.Add(wx.StaticLine(parent, -1, style=wx.LI_HORIZONTAL), 0,
                  wx.EXPAND | wx.BOTTOM, 5)
        sizer.Add(tb_sizer, 0, wx.ALIGN_RIGHT | wx.EXPAND)
        sizer.Add(wx.StaticLine(parent, -1, style=wx.LI_HORIZONTAL), 0,
                  wx.EXPAND | wx.BOTTOM, 5)

    #-------------------------------------------------------------------------
    #  Handles the user requesting that columns not be sorted:
    #-------------------------------------------------------------------------

    def on_savedata(self):
        """ Handles the user requesting that the data of the function \
            is to be saved.
        """
        import os
        dlg = FileDialog(parent=self.control,
                         title='Export function data',
                         default_directory=os.getcwd(),
                         default_filename="", wildcard='*.csv',
                         action='save as')
        if dlg.open() == OK:
            path = dlg.path

            print("Saving data to", path, "...")
            try:

                factory = self.factory
                plotitem = factory.plotitem
                x_values = getattr(self.object, plotitem.index)
                y_values = getattr(self.object, plotitem.value_list)

            except:
                print("Error saving!")
                raise
            print("Plot saved.")
        return

    def on_savefig(self):
        """ Handles the user requesting that the image of the function \
            is to be saved.
        """
        import os
        dlg = FileDialog(parent=self.control,
                         title='Save as image',
                         default_directory=os.getcwd(),
                         default_filename="", wildcard=WILDCARD,
                         action='save as')
        if dlg.open() == OK:
            path = dlg.path

            print("Saving plot to", path, "...")
            try:

                # Now we create a canvas of the appropriate size and
                # ask it to render our component.
                # (If we wanted to display this plot in a window, we
                # would not need to create the graphics context ourselves;
                # it would be created for us by the window.)
                self._plot.bounds = [500, 300]
                self._plot.padding = 50
                plot_gc = PlotGraphicsContext(self._plot.outer_bounds)
                print(self._plot.outer_bounds)
                plot_gc.render_component(self._plot)

                # Finally, we tell the graphics context to save itself to
                # disk as an image.
                plot_gc.save(path)

            except:
                print("Error saving!")
                raise
            print("Plot saved.")
        return

    #-------------------------------------------------------------------------
    #  Disposes of the contents of an editor:
    #-------------------------------------------------------------------------

    def _destroy_plot(self):
        plot_fn_list = [self._plot_radiusfn, self._plot_axeslabels]
        if self.plot_unitcircle_flag:
            plot_fn_list += [self._plot_unitcircle]
            self.plot_unitcircle_flag = False
        for plot_fn in plot_fn_list:
            if self._container and plot_fn:
                plot = plot_fn
                del plot.index._data
                del plot.index._cached_mask
                del plot.value._data
                del plot.value._cached_mask
                self._container.remove(plot)
                plot_fn = None
                plot.index = None
                plot.value = None
        return

    #-------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #-------------------------------------------------------------------------

    def update_editor(self):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """

        factory = self.factory
        plotitem = factory.plotitem

        # Remove the old plot
        if self._plot_radiusfn is not None:
            self._destroy_plot()

        try:
            theta_arr = getattr(self.object, plotitem.index)
            radius_arr = getattr(self.object, plotitem.value_list[0])
            plotrange_min = getattr(self.object, plotitem.value_list[1])
            plotrange_max = getattr(self.object, plotitem.value_list[2])
            frac_noplot = getattr(self.object, plotitem.value_list[3])

        except:
            self._container.request_redraw()
            return

        # check plausibility
        if not ((array([plotrange_min]) <= radius_arr).all() and
                (radius_arr <= array([plotrange_max])).all()):
            print("Note: some value out of range! Compare plotranges with \
                   indicated values for Radius_min and Radius_max ")

        if self.validate(plotrange_min, plotrange_max):
            # plot unitcircle if zero is in plotrange
            unitcircle_arr = _get_unitcircle_arr(theta_arr, frac_noplot,
                                                 plotrange_min, plotrange_max)

            bool_arr_frac_noplot = (unitcircle_arr == array([frac_noplot]))
            bool_arr_1 = (unitcircle_arr == array([1.]))

            if (plotrange_min == 0. or plotrange_max == 0.) or  \
                (bool_arr_frac_noplot.all() == False and
                 bool_arr_1.all() == False):
                self.plot_unitcircle_flag = True
                plot_unitcircle = self._create_polar_plot(plotitem,
                                                          (unitcircle_arr,
                                                           theta_arr),
                                                          color='red',
                                                          frac_noplot=frac_noplot,
                                                          width=5.0)

                self._plot_unitcircle = plot_unitcircle

#                label = DataLabel(component=plot_unitcircle, \
# data_point=(unitcircle_arr[0],theta_arr[0]), # data_point=(x[40], y[40]),
#                                  padding=40,
#                                  label_format = 'What shall I write here?',
#                                  bgcolor = "transparent",
#                                  border_visible=False)
#                self._plot_unitcircle.overlays.append(label)

                self._container.add(plot_unitcircle)
                self._container.request_redraw()

                print('container', self._container.get_preferred_size())

                print('width for Unitcircle', plot_unitcircle.width)

            # plot a single point at the position of 'plotrange_min'
            radius_axeslabels = array([frac_noplot])
            theta_axeslabels = array([0])
            plot_axeslabels = self._create_polar_plot(plotitem,
                                                      (radius_axeslabels,
                                                       theta_axeslabels),
                                                      color='red',
                                                      frac_noplot=frac_noplot,
                                                      width=5.0)


# using 'DataLabel' label
#            label = DataLabel(component=plot_axeslabels,
#                              label_position="top left", padding=40,
#                              bgcolor = "lightgray",
#                              border_visible=False)
#


# using 'PlotLabel'
#            pltlblcenter = PlotLabel("0",component=plot_axeslabels)
#            self._container.overlays.append(pltlblcenter)
#            self._plot_axeslabels = plot_axeslabels


#            label = DataLabel(component=self._plot_axeslabels, \
# data_point=(5,0.5), # data_point=(x[40], y[40]),
#                          label_position=(200,200),
#                          label_format = 'What shall I write here?',
#                          bgcolor = "transparent",
#                          border_visible=False)
#            self._plot_axeslabels.overlays.append(label)

            x_center = factory.width / 2.0
            y_center = factory.height / 2.0

            print(('factory.width', factory.width))
            print(('factory.height', factory.height))

            x_frac_min = frac_noplot * factory.height / 2
            x_frac_max = factory.height / 2

            x_pnt_min = x_center + x_frac_min
            x_pnt_max = x_center + x_frac_max
            y_pnt = y_center

            print(('x_center=', x_center))
            print(('y_center=', y_center))

            print(('x_frac_min=', x_frac_min))
            print(('x_frac_max=', x_frac_max))

            print(('x_pnt_min=', x_pnt_min))
            print(('x_pnt_max=', x_pnt_max))

#            txt_plotrange_min = TextBoxOverlay(text = plotrange_min.__str__(), \
#                                alternate_position = (x_pnt_min,y_pnt) )
            txt_plotrange_min = TextBoxOverlay(text=plotrange_min.__str__(),
                                               alternate_position=(285, 189))
            plot_axeslabels.overlays.append(txt_plotrange_min)

            txt_plotrange_max = TextBoxOverlay(text=plotrange_max.__str__(),
                                               alternate_position=(285 * 2, 189))
            plot_axeslabels.overlays.append(txt_plotrange_max)

            self._container.add(plot_axeslabels)
            self._container.request_redraw()

            # function plot
            radius_arr_plt = _get_radius_arr_plt(radius_arr, frac_noplot,
                                                 plotrange_min, plotrange_max)

            plot_radiusfn = self._create_polar_plot(plotitem,
                                                    (radius_arr_plt,
                                                     theta_arr),
                                                    frac_noplot=frac_noplot,
                                                    width=5.0,
                                                    index_sort="ascending")

            self._set_basic_properties(plot_radiusfn, plotitem)

            self._plot_radiusfn = plot_radiusfn
            self._container.add(plot_radiusfn)
            self._container.request_redraw()
        else:
            self._plot_radiusfn = None
            self._plot_axeslabels = None


#        plot_unitcircle.tools.append(PanTool(plot_unitcircle))
#        zoom = SimpleZoom(plot_unitcircle, tool_mode="box", always_on=False)
#        plot_unitcircle.overlays.append(zoom)
#

#        label = DataLabel(component=plot_unitcircle, \
      #                        data_point=(unitcircle_values, theta_values),
#                              label_position="top left", padding=40,
#                              bgcolor = "lightgray",
#                              border_visible=False)
#        plot_unitcircle.overlays.append(label)

#        tool = DataLabelTool(label, drag_button="right", auto_arrow_root=True)
#        label.tools.append(tool)

    def _update_data(self):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        self.update_editor()

    #-------------------------------------------------------------------------
    #  Creates the table editing tool bar:
    #-------------------------------------------------------------------------

    def _create_polar_plot(self, plotitem, data,
                           color='black',
                           width=1.0,
                           dash="solid",
                           grid="dot",
                           value_mapper_class=PolarMapper,
                           frac_noplot=0.3,
                           **kwargs):
        if (type(data) != ndarray) and (len(data) == 2):
            data = transpose(array(data))

        r_data, t_data = transpose(data)
        index_data = r_data * cos(t_data)
        value_data = r_data * sin(t_data)

        index = ArrayDataSource(index_data, sort_order='ascending')
        # Typically the value data is unsorted
        value = ArrayDataSource(value_data)

        index_range = DataRange1D()
        index_range.add(index)
        index_mapper = PolarMapper(range=index_range)

        value_range = DataRange1D()
        value_range.add(value)
        value_mapper = value_mapper_class(range=value_range)

        plot = MFnPolarLineRenderer(index=index, value=value,
                                    index_mapper=index_mapper,
                                    value_mapper=value_mapper,
                                    color=color,
                                    line_width=width,
                                    line_style=dash,
                                    grid_style=grid,
                                    frac_noplot=frac_noplot,
                                    origin_axis_visible=True)
        return plot

    def _set_basic_properties(self, plot, plotitem):
        for attr in ("color", "bgcolor", "border_visible", "border_width",
                     "border_color"):
                #"border_dash", "border_color"):
            setattr(plot, attr, getattr(plotitem, attr))
        return


class MFnPolarPlotEditorToolbar (HasPrivateTraits):

    """ Toolbar displayed in table editors.
    """

    #-------------------------------------------------------------------------
    #  Trait definitions:
    #-------------------------------------------------------------------------

    # Do not sort columns:
    save_data = Instance(Action,
                         {'name':    'Save as data',
                          'tooltip': 'Save the function values',
                          'action':  'on_savedata',
                          'enabled': True,
                          'image':   ImageResource('table_no_sort.png')})

    # Move current object up one row:
    save_fig = Instance(Action,
                        {'name':    'Save as fig',
                         'tooltip': 'Save as figure',
                         'action':  'on_savefig',
                         'enabled': True,
                         'image':   ImageResource('table_move_down.png')})

    # The table editor that this is the toolbar for:
    editor = Instance(MFnPolarPlotEditor)

    # The toolbar control:
    control = Any

    #-------------------------------------------------------------------------
    #  Initializes the toolbar for a specified window:
    #-------------------------------------------------------------------------

    def __init__(self, parent=None, **traits):
        super(MFnPolarPlotEditorToolbar, self).__init__(**traits)
        factory = self.editor.factory

        actions = [self.save_data, self.save_fig]
        toolbar = ToolBar(image_size=(16, 16),
                          show_tool_names = False,
                          show_divider = False,
                          *actions)
        self.control = toolbar.create_tool_bar(parent, self)
        self.control.SetBackgroundColour(parent.GetBackgroundColour())

        # fixme: Why do we have to explictly set the size of the toolbar?
        #        Is there some method that needs to be called to do the
        #        layout?
        self.control.SetSize(wx.Size(23 * len(actions), 16))

    #-------------------------------------------------------------------------
    #  PyFace/Traits menu/toolbar controller interface:
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    #  Adds a menu item to the menu bar being constructed:
    #-------------------------------------------------------------------------

    def add_to_menu(self, menu_item):
        """ Adds a menu item to the menu bar being constructed.
        """
        pass

    #-------------------------------------------------------------------------
    #  Adds a tool bar item to the tool bar being constructed:
    #-------------------------------------------------------------------------

    def add_to_toolbar(self, toolbar_item):
        """ Adds a toolbar item to the too bar being constructed.
        """
        pass

    #-------------------------------------------------------------------------
    #  Returns whether the menu action should be defined in the user interface:
    #-------------------------------------------------------------------------

    def can_add_to_menu(self, action):
        """ Returns whether the action should be defined in the user interface.
        """
        return True

    #-------------------------------------------------------------------------
    #  Returns whether the toolbar action should be defined in the user
    #  interface:
    #-------------------------------------------------------------------------

    def can_add_to_toolbar(self, action):
        """ Returns whether the toolbar action should be defined in the user
            interface.
        """
        return True

    #-------------------------------------------------------------------------
    #  Performs the action described by a specified Action object:
    #-------------------------------------------------------------------------

    def perform(self, action, action_event=None):
        """ Performs the action described by a specified Action object.
        """
        getattr(self.editor, action.action)()


#from etsproxy.chaco.polar_line_renderer import PolarLineRenderer

# class MFnPolarLineRenderer(PolarLineRenderer):
class MFnPolarLineRenderer(AbstractPlotRenderer):

    """ A renderer for polar line plots.
    """
    #------------------------------------------------------------------------
    # Appearance-related traits
    #------------------------------------------------------------------------

    # The color of the origin axis.
    origin_axis_color_ = (0, 0, 0, 1)
    # The width of the origin axis.
    origin_axis_width = 1.0
    # The origin axis is visible.
    origin_axis_visible = True
    # The grid is visible.
    grid_visible = True
    # The color of the line.
    color = black_color_trait
    # The width of the line.
    line_width = Float(1.0)
    # The style of the line.
    line_style = LineStyle("solid")
    # The style of the grid lines.
    grid_style = LineStyle("dot")

    frac_noplot = Float(0.3)

    def _gather_points(self):
        """
        Collects the data points that are within the plot bounds and caches them
        """
        # This is just a stub for now.  We should really find the lines only
        # inside the screen range here.

        x = self.index.get_data()
        y = self.value.get_data()
        rad = min(self.width / 2.0, self.height / 2.0)
        sx = x * rad + self.x + self.width / 2.0
        sy = y * rad + self.y + self.height / 2.0

        points = transpose(array((sx, sy)))
        self._cached_data_pts = points
        self._cache_valid = True
        return

    def _render(self, gc, points):
        """ Actually draw the plot.
        """
        gc.save_state()

        gc.set_antialias(True)
        self._draw_default_axes(gc)
        self._draw_default_grid(gc)
        if len(points) > 0:
            gc.clip_to_rect(self.x, self.y, self.width, self.height)
            gc.set_stroke_color(self.color_)
            gc.set_line_width(self.line_width)
            gc.set_line_dash(self.line_style_)

            gc.begin_path()
            gc.lines(points)
            gc.stroke_path()

        gc.restore_state()

        return

    def _draw_plot(self, *args, **kw):
        """ Draws the 'plot' layer.
        """
        # Simple compatibility with new-style rendering loop
        return self._draw_component(*args, **kw)

    def _draw_component(self, gc, view_bounds=None, mode='normal'):
        """ Renders the component. 
        """
        self._gather_points()
        self._render(gc, self._cached_data_pts)

    def _draw_default_axes(self, gc):
        if not self.origin_axis_visible:
            return
        gc.save_state()
        gc.set_stroke_color(self.origin_axis_color_)
        gc.set_line_width(self.origin_axis_width)
        gc.set_line_dash(self.grid_style_)
        x_data, y_data = transpose(self._cached_data_pts)
        x_center = self.x + self.width / 2.0
        y_center = self.y + self.height / 2.0

        # number of divisions used to divide the cirlce radially
        # (equals the number of axes to be plotted)
        n_axes = 2
        for theta in range(n_axes * 2):
            r = min(self.width / 2.0, self.height / 2.0)
            x = r * cos(theta * pi / n_axes) + x_center
            y = r * sin(theta * pi / n_axes) + y_center
            data_pts = array([[x_center, y_center], [x, y]])
            start, end = data_pts
            gc.move_to(int(start[0]), int(start[1]))
            gc.line_to(int(end[0]), int(end[1]))
            gc.stroke_path()

        gc.restore_state()
        return

    def _draw_default_grid(self, gc):
        if not self.grid_visible:
            return
        gc.save_state()
        gc.set_stroke_color(self.origin_axis_color_)
        gc.set_line_width(self.origin_axis_width)
        gc.set_line_dash(self.grid_style_)
        x_data, y_data = transpose(self._cached_data_pts)
        x_center = self.x + self.width / 2.0
        y_center = self.y + self.height / 2.0

        print(('self.x', self.x))
        print(('self.y', self.y))

        print(('2*****self.width/2.0', self.width / 2.0))
        print(('2****self.height/2.0', self.height / 2.0))

        print(('x_center', x_center))
        print(('y_center', y_center))

        rad_one = min(self.width / 2.0, self.height / 2.0)
        rad_unitcircle_plt = 0.3

        # number of divisions used to divide the cirlce tangentially
        ndivs_grid = 4
        for i in range(ndivs_grid + 1):
            plotrange_max = 1.0
            plotrange_min = 0.0
            rad_i = plotrange_min + \
                (plotrange_max - plotrange_min) / ndivs_grid * i
            rad_i_plt = coord_trans_plt(rad_i, self.frac_noplot,
                                        plotrange_min, plotrange_max) * rad_one
            gc.move_to(self.x, self.y)
            gc.arc(x_center, y_center, rad_i_plt, 0, 2 * pi)
            gc.stroke_path()

        gc.restore_state()
        return


# EOF



'''
Created on Sep 4, 2012

@author: rch
'''
from traits.api import \
    Property, cached_property, Int, \
    on_trait_change

from traitsui.api import \
    View, Item, VGroup, Group

from bmcs_beam.mxn.material_types import \
    MTReinfFabric

from .reinf_layout_component import \
    ReinfLayoutComponent, \
    STATE_LAW_AND_GEOMETRY_CHANGE, \
    STATE_AND_GEOMETRY_CHANGE

from .reinf_layout_component_tex_layer import \
    RLCTexLayer

from .reinf_fabric_handler import \
    FabricHandler

from bmcs_beam.mxn.utils import \
    KeyRef

import numpy as np

class RLCTexUniform(ReinfLayoutComponent):

    def __init__(self, *args, **metadata):
        if not metadata.get('material', None):
            metadata['material'] = 'default_fabric'
        super(RLCTexUniform, self).__init__(**metadata)

    n_layers = Int(12, auto_set=False, enter_set=True, geo_input=True)
    '''Total number of reinforcement layers [-]
    '''

    material = KeyRef('default_fabric', db=MTReinfFabric.db)

    def convert_eps_u_2_lo(self, eps_up):
        '''Convert the strain in the lowest reinforcement layer at failure
        to the strain at the bottom of the cross section'''
        eps_tex_u = self.material_law_.eps_u
        height = self.matrix_cs.geo.height
        return eps_up + (eps_tex_u - eps_up) / self.z_ti_arr[0] * height

    converted_eps_lo_2_u = Property()
    def _get_converted_eps_lo_2_u(self):
        '''Convert the strain at the bottom of the cross section to the strain
        in the lowest reinforcement layer at failure'''
        eps_lo = self.state.eps_lo
        eps_up = self.state.eps_up
        height = self.matrix_cs.geo.height
        return (eps_up + (eps_lo - eps_up) / height * self.z_ti_arr[0])

    #===========================================================================
    # Distribution of reinforcement
    #===========================================================================

    s_tex_z = Property(depends_on='+geo_input,matrix_cs.geo.changed')
    '''spacing between the layers [m]'''
    @cached_property
    def _get_s_tex_z(self):
        return self.matrix_cs.geo.height / (self.n_layers + 1)

    z_ti_arr = Property(depends_on='+geo_input,matrix_cs.geo.changed')
    '''property: distance of each reinforcement layer from the top [m]:
    '''
    @cached_property
    def _get_z_ti_arr(self):
        return np.array([ self.matrix_cs.geo.height - (i + 1) * self.s_tex_z
                         for i in range(self.n_layers) ],
                      dtype=float)

    zz_ti_arr = Property(depends_on='+geo_input,matrix_cs.geo.changed')
    '''property: distance of reinforcement layers from the bottom
    '''
    def _get_zz_ti_arr(self):
        return self.matrix_cs.geo.height - self.z_ti_arr

    #===========================================================================
    # Discretization conform to the tex layers
    #===========================================================================

    layer_lst = Property(depends_on='+geo_input,matrix_cs.geo.changed,+law_input,material_law')
    '''List of reinforcement layers
    '''
    @cached_property
    def _get_layer_lst(self):
        lst = []
        for i in range(self.n_layers):
            lst.append(RLCTexLayer(state=self.state, matrix_cs=self.matrix_cs,
                                     z_coord=self.zz_ti_arr[i],
                                     material=self.material,
                                     material_law=self.material_law,
                                     ))
        return lst

    @on_trait_change('eps_changed')
    def notify_eps_change(self):
        for layer in self.layer_lst:
            layer.eps_changed = True

    N = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Get the resulting normal force.
    '''
    @cached_property
    def _get_N(self):
        N = 0.
        for i in range(self.n_layers):
            N += self.layer_lst[i].N
        return N

    M = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Get the resulting moment.
    '''
    @cached_property
    def _get_M(self):
        M = 0.
        for i in range(self.n_layers):
            M += self.layer_lst[i].M
        return M

    #===========================================================================
    # UI-related functionality
    #===========================================================================

    node_name = 'Uniform textile layers'

    def plot_geometry(self, ax, clr='DarkOrange'):
        '''Plot geometry'''
        for i in range(self.n_layers):
            self.layer_lst[i].plot_geometry(ax, clr=clr)

    def plot_eps(self, ax):
        '''Plot strains'''
        for i in range(self.n_layers):
            self.layer_lst[i].plot_eps(ax)

    def plot_sig(self, ax):
        '''Plot stresses'''
        for i in range(self.n_layers):
            self.layer_lst[i].plot_sig(ax)

    tree_view = View(VGroup(
                      Group(
                      Item('n_layers'),
                      label='Geometry'
                      ),
                      Group(
                      Item('material'),
                      Item('material_law'),
                      springy=True,
                      label='Fabric material',
                      ),
                      springy=True,
                      ),
                resizable=True,
                handler=FabricHandler(),
                buttons=['OK', 'Cancel']
                )


if __name__ == '__main__':
    Layers = RLCTexUniform()
    Layers.configure_traits()



'''
Created on 1. 4. 2014

@author: Vancikv

Standard tree view with the default database node
and added calibration node
'''

from bmcs_beam.mxn.view import \
    MxNTreeView

from bmcs_beam.mxn.use_cases import \
    UseCaseContainer

ucc = UseCaseContainer()
ucc.use_case_to_add = 'calibration'
ucc.add_use_case = True

uc_view = MxNTreeView(root=ucc)
uc_view.configure_traits()



'''
Created on Nov 21, 2013

@author: rch
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCTexUniform

import numpy as np

def test_cross_section_mn():
    '''Test the moment and normal force calculated for a cross section.
    '''
    cp = CrossSection(reinf=[RLCTexUniform(n_layers=3, material='default_fabric', material_law='fbm')],
                         matrix_cs=MatrixCrossSection(geo=MCSGeoRect(width=0.1, height=0.05),
                                                      n_cj=20, material_law='constant', material='default_mixture'),
                         eps_lo=0.014,
                         eps_up=-0.0033,
                         )

    cp.reinf_components_with_state[0].material_law_.set(sig_tex_u=1216., eps_u=0.014, m=0.5)
    cp.reinf_components_with_state[0].material_.set(s_0=0.00416, A_roving=0.461)

    assert np.allclose([cp.M, cp.N], [1.14513592334, -22.1303533699])
    cp.reinf[0].n_layers = 5

    assert np.allclose([cp.M, cp.N], [1.29225385264, -6.60917224146])
    cp.eps_lo = 0.010

    assert np.allclose([cp.M, cp.N], [1.52939155655, -28.4691640432])

if __name__ == '__main__':
    test_cross_section_mn()



'''
Created on 23. 4. 2014

@author: Vancikv
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCTexUniform

from bmcs_beam.mxn.matrix_laws import \
    MatrixLawBase, MatrixLawBlock, MatrixLawQuad

from bmcs_beam.mxn.material_types import \
    MTMatrixMixture

import numpy as np

import pickle

from bmcs_beam.mxn.utils import \
    get_outfile

if MTMatrixMixture.db.get('mixture-test', None):
    del MTMatrixMixture.db['mixture-test']
MTMatrixMixture.db['mixture-test'] = MTMatrixMixture(f_ck=53.7,
                                                eps_c_u=0.0033,
                                                mtrl_laws={'constant':
                                                                MatrixLawBlock(f_ck=55.7, eps_c_u=0.0033,
                                                                    high_strength_level=50.0, E_c=28e+3),
                                                                'quad':
                                                                MatrixLawQuad(f_ck=55.7, eps_c_u=0.0033,
                                                                    high_strength_level=50.0, E_c=28e+3),
                                         })

def test_matrix_mixture_states():
    '''Test the moment and normal force calculated for a cross section
    with changing matrix law.
    '''
    cp = CrossSection(reinf=[RLCTexUniform(n_layers=6, material='default_fabric', material_law='fbm')],
                         matrix_cs=MatrixCrossSection(geo=MCSGeoRect(width=0.1, height=0.05),
                                                      n_cj=20, material_law='constant', material='default_mixture'),
                         eps_lo=0.014,
                         eps_up=-0.0033,
                         )

    cp.reinf_components_with_state[0].material_law_.set(sig_tex_u=1216., eps_u=0.014, m=0.5)
    cp.reinf_components_with_state[0].material_.set(s_0=0.00416, A_roving=0.461)

    assert np.allclose([cp.M, cp.N], [1.3465387287796249, 2.3335097542460943])
    cp.matrix_cs_with_state.material = 'mixture-test'

    object_file = get_outfile(folder_name='.mxn',
                             file_name='test09_reinf_law.pkl')
    pickle.dump(cp, open(object_file, 'wb'), 1)
    assert np.allclose([cp.M, cp.N], [1.3140950375716971, 3.8701260913315991])

    MTMatrixMixture.db['mixture-test'].f_ck = 70.0
    assert np.allclose([cp.M, cp.N], [1.5155334535680236, -5.0522452437834104])
    MTMatrixMixture.db['mixture-test'].f_ck = 53.7
    assert np.allclose([cp.M, cp.N], [1.3140950375716971, 3.8701260913315991])

    cp.matrix_cs_with_state.material_law = 'quad'
    assert np.allclose([cp.M, cp.N], [0.98987130205727225, 19.838093373553274])
    MTMatrixMixture.db['mixture-test'].mtrl_laws['quad'].E_c = 20000.
    assert np.allclose([cp.M, cp.N], [0.90521419096946409, 24.022573435893499])
    MTMatrixMixture.db['mixture-test'].mtrl_laws['quad'].f_ck = 60.0
    assert np.allclose([cp.M, cp.N], [0.95059541049994045, 22.016807281563626])

    loaded_cp = pickle.load(open(object_file, 'rb'))
    assert np.allclose([loaded_cp.M, loaded_cp.N], [1.3140950375716971, 3.8701260913315991])
    MTMatrixMixture.db['mixture-test'].f_ck = 70.0
    assert np.allclose([loaded_cp.M, loaded_cp.N], [1.5155334535680236, -5.0522452437834104])

    cp.matrix_cs_with_state.material_law = 'constant'
    assert np.allclose([cp.M, cp.N], [1.5155334535680236, -5.0522452437834104])
    cp.matrix_cs_with_state.material_law = 'quad'
    MTMatrixMixture.db['mixture-test'].f_ck = 53.7
    assert np.allclose([cp.M, cp.N], [0.98987130205727203, 19.838093373553281])

if __name__ == '__main__':
    test_matrix_mixture_states()



'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Property, cached_property

import numpy as np

from bmcs_beam.mxn.mfn import MFnLineArray

from .matrix_law_base import \
    MatrixLawBase


class MatrixLawQuadratic(MatrixLawBase):

    '''Effective crack bridge Law using a cubic polynomial.'''

    mfn = Property(depends_on='+input')

    @cached_property
    def _get_mfn(self):
        '''quadratic stress-strain-diagram of the concrete
        '''
        # (for all concretes up to f_cm=88 N/mm2) #max epislon_c1u
        f_cm = self.f_ck + 8
        E_tan = 22. * (f_cm / 10) ** 0.3 * 1000.
        eps_c1 = min(0.7 * f_cm ** 0.31, 2.8) / 1000.  # EC2
        # @todo: with constant value this yields negative values for strains close to 'eps_c1u'
# eps_c1 = 0.0022 #Brockmann
        E_sec = f_cm / eps_c1

        if self.f_ck <= self.high_strength_level:
            eps_c1u = self.eps_c_u
            eps_arr = np.linspace(0., eps_c1u, num=100.)

        elif self.f_ck > self.high_strength_level:
            eps_c1u = (2.8 + 27. * (((98. - f_cm) / 100.) ** 4.)) / 1000.
            eps_arr = np.linspace(0., eps_c1u, num=100.)

        k = E_tan / E_sec
        sig_c_arr = ((k * eps_arr / eps_c1 - (eps_arr / eps_c1) **
                      2.) / (1. + (k - 2.) * eps_arr / eps_c1)) * f_cm

        xdata = eps_arr
        ydata = sig_c_arr

        return MFnLineArray(xdata=xdata, ydata=ydata)



#-------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Created on May 25, 2009 by Rostislav Chudoba
#
#-------------------------------------------------------------------------

from matplotlib.figure import Figure
from traits.api import Instance, Int
from traitsui.basic_editor_factory import BasicEditorFactory

from .mfn_plot_adapter import MFnPlotAdapter
from util.traits.editors import MPLFigureEditor


class _MFnMatplotlibEditor(MPLFigureEditor):

    # @todo This is not used here, either activate or delete
    # scrollable  = True

    # the plot representation in matplotlib
    figure = Instance(Figure(facecolor=MFnPlotAdapter().padding_bg_color), ())

    # adjustable parameters
    adapter = Instance(MFnPlotAdapter)

    border_size = Int(0)
    # @todo faezeh please make the mapping from the human readable color
    #

    def init(self, parent):

        super(_MFnMatplotlibEditor, self).init(parent)
        factory = self.factory
        self.adapter = factory.adapter

    def update_editor(self):
        figure = self.figure
        axes = figure.add_subplot(111)
        canvas = figure.canvas
        if canvas is None:
            pass
        else:
            figure.delaxes(axes)
            self._refresh_plot()
            figure.canvas.draw()

    def _create_canvas(self, parent):
        """ Create the MPL canvas. """
        # The panel lets us add additional controls.
        a = self.adapter
        fig = self.figure

        panel = super(_MFnMatplotlibEditor, self)._create_canvas(parent)

#         panel = wx.Panel(parent, -1, style=wx.CLIP_CHILDREN)
#         sizer = wx.BoxSizer(wx.VERTICAL)
#         panel.SetSizer(sizer)
#
#         # matplotlib commands to create a canvas
#         mpl_control = FigureCanvas(panel, -1, fig)
#         toolbar = NavigationToolbar2Wx(mpl_control)
#
#         sizer.Add(toolbar, 0, wx.EXPAND)
#         sizer.Add(mpl_control, 1, wx.LEFT | wx.TOP | wx.GROW)

        if a.max_size:
            self.figure.canvas.SetMaxSize(a.max_size)
        if a.min_size:
            self.figure.canvas.SetMinSize((a.min_size))

        if a.padding:
            for side, pad in list(a.padding.items()):
                setattr(self.figure.subplotpars, side, pad)

        return panel

    def _refresh_plot(self):

        figure = self.figure

        mfn_line = self.value
        x = mfn_line.xdata
        y = mfn_line.ydata
        axes = figure.add_subplot(111)

        a = self.adapter
        if a.var_x != '':
            label_x = getattr(self.object, a.var_x)
        else:
            label_x = a.label_x

        if a.var_y != '':
            label_y = getattr(self.object, a.var_y)
        else:
            label_y = a.label_y

        styles_m = list(a.line_style.values())
        #  line_color = self.line_color_matplotlib[ a.line_color ]
        line_color = a.line_color[0]
        line_style = styles_m[0]
        # line_style = self.line_style_matplotlib[ a.line_style  ]

        axes.plot(x, y, color=line_color, linewidth=2.,
                  linestyle=line_style)
        axes.set_xlabel(label_x, weight='semibold')
        axes.set_ylabel(label_y, weight='semibold')
        axes.set_title(a.title, size='large', color=a.title_color,
                       weight='bold', position=(.5, 1.03))
        axes.set_axis_bgcolor(color=a.bgcolor)
        axes.ticklabel_format(scilimits=a.scilimits)
        axes.grid(color='gray', linestyle='--', linewidth=0.1, alpha=0.4)
        axes.set_xscale(a.xscale)
        axes.set_yscale(a.yscale)

# class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
#
#    def _init_toolbar(self):
#        _NTB2_DSAVE    = wx.NewId()
#        self.AddSimpleTool(_NTB2_DSAVE, _load_bitmap('filesave.png'),
#                           'Data Save', 'Save plot data to file')
#        bind(self, wx.EVT_TOOL, self.save, id=_NTB2_DSAVE)


class MFnMatplotlibEditor(BasicEditorFactory):

    klass = _MFnMatplotlibEditor

    adapter = Instance(MFnPlotAdapter)

    def _adapter_default(self):
        return MFnPlotAdapter()



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Created on May 25, 2009 by Rostislav Chudoba
#
#-------------------------------------------------------------------------------

from traits.api import \
    HasTraits, HasPrivateTraits, Color, Str, Int, Float, Enum, List, \
    Dict, Bool, Instance, Any, Font, Event, Property, Interface, \
    on_trait_change, cached_property, implements, Color, Tuple
           
#-------------------------------------------------------------------------------
#  'MFnPlotAdapter' class:
#-------------------------------------------------------------------------------

class MFnPlotAdapter ( HasPrivateTraits ):
    
    """ The base class for adapting function implementation in order to plot
    it in a plotting toolkit (either Chaco or Matplotlib)
    """
    
    # label on the x axis (only used when var_x empty)
    #
    label_x = Str( '' )
    
    # label on the y axis (only used when var_y empty)
    label_y = Str( '' )

    # title of the diagram
    #
    title = Str('')
    
    # color of the title
    title_color = Str('black')
    
    # when specified take the label of the axis from the value 
    # of the variable trait in object
    #
    var_x = Str('')
    
    # when specified take the label of the axis from the value 
    # of the variable trait in object
    #
    var_y = Str('')
    
    # label for line in legend
    #
    line_label = Str('')
    
    # @todo - further attributes to be made available
    
    # limits of number positions for switching into scientific notation 1eXX
    scilimits = Tuple(-3.,4.)
    
    # Plot properties
    
    line_color =  List(['blue', 
                        'red',
                        'green',
                        'black',
                        'magenta',
                        'yellow',
                        'white',
                        'cyan'
                        ]) 

    #rr: is the dictionary necessary here?
    #why not a list of keys? chaco as well mpl can work with the key 
    line_style = Dict({'solid' : '-',
                  'dash' : '.',
                  'dot dash' : '-.',
                  'dot' : ':',
                 })

    #horisontal axis scale ('log', 'symlog', 'linear')    
    xscale = Str('linear')
    
    #vertical axis scale ('log', 'symlog', 'linear')    
    yscale = Str('linear')    
    
    # linewidth
    line_width = List(2.0)
    
    # color of the plotting background
    bgcolor = Str( 'white' )
  
    # Border, padding properties
    border_visible = Bool( False )

    # @todo - unused - padding defined below - should be used in Chaco - 
    # in Matplotlib it is working already [Faezeh, Check please]
    border_width =  Int( 0 )
    
    # @todo does this work in both Mpl and Chaco? [Faeseh Check please]
    #
    padding_bg_color = Str( 'white' )
    
    # labels in legend
    
    # @todo - no labels here - the defaults should make sense - are they active?
    # 
    legend_labels = Tuple()
    
    # maximum size - None means arbitrarily resizable
    #
    max_size = Tuple()
    
    # minimum size
    #
    min_size = Tuple( ( 200, 200 ) )

    # padding
    
    padding = Dict( { 'left'   : 0.1,
                      'bottom' : 0.1,
                      'right'  : 0.9,
                      'top'    : 0.9 } )
    
    # No of values displayed on the x axis
    xticks = Int(0)
    
    # No of values displayed on the y axis
    yticks = Int(0)

#-------------------------------------------------------------------------------
#  'MFnPlotAdapter' class:
#-------------------------------------------------------------------------------

class MFnMultiPlotAdapter ( MFnPlotAdapter ):
    
    """ The base class for adapting function implementation in order to plot
    it in a plotting toolkit (either Chaco or Matplotlib)
    """

    # rr: test only, needs to be written in a more robust way
    mline_style = List( 100 * ['solid'] )
    
    mline_color = List( 100 * ['black'] )
    
    mline_width = List(100 * [2.0])

    
    
    
    


'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property, Array, Int

import numpy as np

from .reinf_law_base import \
    ReinfLawBase

class ReinfLawPiecewiseLinear(ReinfLawBase):
    '''Effective crack bridge Law using a piecewise linear function.'''

    sig_level_arr = Array(float, input=True)
    def _sig_level_arr_default(self):
        E_eff = self.sig_tex_u / self.eps_u
        return E_eff * self.eps_arr

    sig_tex_u = Float(800.0, enter_set=True, auto_set=False, input=True)
    eps_u = Float(0.01, enter_set=True, auto_set=False, input=True)
    n_eps = Int(2, enter_set=True, auto_set=False, input=True)

    eps_fraction_arr = Property(depends_on='n_eps')
    @cached_property
    def _get_eps_fraction_arr(self):
        return np.linspace(0.0, 1.0, self.n_eps)

    cnames = ['eps_u', 'sig_level_arr']

    u0 = Property(depends_on='eps_u, sig_tex_u, eps_fraction_list')
    @cached_property
    def _get_u0(self):
        return self.sig_level_arr[1:]

    eps_arr = Array(float)
    def _eps_arr_default(self):
        return self.eps_fraction_arr * self.eps_u

    sig_arr = Array(float)
    def _sig_arr_default(self):
        return self.sig_level_arr

    def set_sig_eps_arr(self, eps_arr, sig_arr):
        self.eps_arr = eps_arr
        self.sig_arr = sig_arr

ReinfLawBase.db.constants['piecewise_linear-default'] = ReinfLawPiecewiseLinear()
